"""
Test lldb-dap logpoints feature.
"""


import dap_server
import shutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase
import os


class TestDAP_logpoints(lldbdap_testcase.DAPTestCaseBase):
    def setUp(self):
        lldbdap_testcase.DAPTestCaseBase.setUp(self)

        self.main_basename = "main-copy.cpp"
        self.main_path = os.path.realpath(self.getBuildArtifact(self.main_basename))

    @skipIfWindows
    def test_logmessage_basic(self):
        """Tests breakpoint logmessage basic functionality."""
        before_loop_line = line_number("main.cpp", "// before loop")
        loop_line = line_number("main.cpp", "// break loop")
        after_loop_line = line_number("main.cpp", "// after loop")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set a breakpoint at a line before loop
        before_loop_breakpoint_ids = self.set_source_breakpoints(
            self.main_path, [before_loop_line]
        )
        self.assertEqual(len(before_loop_breakpoint_ids), 1, "expect one breakpoint")

        self.dap_server.request_continue()

        # Verify we hit the breakpoint before loop line
        self.verify_breakpoint_hit(before_loop_breakpoint_ids)

        # Swallow old console output
        self.get_console()

        # Set two breakpoints:
        # 1. First at the loop line with logMessage
        # 2. Second guard breakpoint at a line after loop
        logMessage_prefix = "This is log message for { -- "
        logMessage = logMessage_prefix + "{i + 3}, {message}"
        [loop_breakpoint_id, post_loop_breakpoint_id] = self.set_source_breakpoints(
            self.main_path,
            [loop_line, after_loop_line],
            [{"logMessage": logMessage}, {}],
        )

        # Continue to trigger the breakpoint with log messages
        self.dap_server.request_continue()

        # Verify we hit the breakpoint after loop line
        self.verify_breakpoint_hit([post_loop_breakpoint_id])

        output = self.get_console()
        lines = output.splitlines()
        logMessage_output = []
        for line in lines:
            if line.startswith(logMessage_prefix):
                logMessage_output.append(line)

        # Verify logMessage count
        loop_count = 10
        self.assertEqual(len(logMessage_output), loop_count)

        message_addr_pattern = r"\b0x[0-9A-Fa-f]+\b"
        message_content = '"Hello from main!"'
        # Verify log message match
        for idx, logMessage_line in enumerate(logMessage_output):
            result = idx + 3
            reg_str = (
                f"{logMessage_prefix}{result}, {message_addr_pattern} {message_content}"
            )
            self.assertRegex(logMessage_line, reg_str)

    @skipIfWindows
    def test_logmessage_advanced(self):
        """Tests breakpoint logmessage functionality for complex expression."""
        before_loop_line = line_number("main.cpp", "// before loop")
        loop_line = line_number("main.cpp", "// break loop")
        after_loop_line = line_number("main.cpp", "// after loop")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set a breakpoint at a line before loop
        before_loop_breakpoint_ids = self.set_source_breakpoints(
            self.main_path, [before_loop_line]
        )
        self.assertEqual(len(before_loop_breakpoint_ids), 1, "expect one breakpoint")

        self.dap_server.request_continue()

        # Verify we hit the breakpoint before loop line
        self.verify_breakpoint_hit(before_loop_breakpoint_ids)

        # Swallow old console output
        self.get_console()

        # Set two breakpoints:
        # 1. First at the loop line with logMessage
        # 2. Second guard breakpoint at a line after loop
        logMessage_prefix = "This is log message for { -- "
        logMessage = (
            logMessage_prefix
            + "{int y = 0; if (i % 3 == 0) { y = i + 3;} else {y = i * 3;} y}"
        )
        [loop_breakpoint_id, post_loop_breakpoint_id] = self.set_source_breakpoints(
            self.main_path,
            [loop_line, after_loop_line],
            [{"logMessage": logMessage}, {}],
        )

        # Continue to trigger the breakpoint with log messages
        self.dap_server.request_continue()

        # Verify we hit the breakpoint after loop line
        self.verify_breakpoint_hit([post_loop_breakpoint_id])

        output = self.get_console()
        lines = output.splitlines()
        logMessage_output = []
        for line in lines:
            if line.startswith(logMessage_prefix):
                logMessage_output.append(line)

        # Verify logMessage count
        loop_count = 10
        self.assertEqual(len(logMessage_output), loop_count)

        # Verify log message match
        for idx, logMessage_line in enumerate(logMessage_output):
            result = idx + 3 if idx % 3 == 0 else idx * 3
            self.assertEqual(logMessage_line, logMessage_prefix + str(result))

    @skipIfWindows
    def test_logmessage_format(self):
        """
        Tests breakpoint logmessage functionality with format.
        """
        before_loop_line = line_number("main.cpp", "// before loop")
        loop_line = line_number("main.cpp", "// break loop")
        after_loop_line = line_number("main.cpp", "// after loop")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set a breakpoint at a line before loop
        before_loop_breakpoint_ids = self.set_source_breakpoints(
            self.main_path, [before_loop_line]
        )
        self.assertEqual(len(before_loop_breakpoint_ids), 1, "expect one breakpoint")

        self.dap_server.request_continue()

        # Verify we hit the breakpoint before loop line
        self.verify_breakpoint_hit(before_loop_breakpoint_ids)

        # Swallow old console output
        self.get_console()

        # Set two breakpoints:
        # 1. First at the loop line with logMessage
        # 2. Second guard breakpoint at a line after loop
        logMessage_prefix = "This is log message for -- "
        logMessage_with_format = "part1\tpart2\bpart3\x64part4"
        logMessage_with_format_raw = r"part1\tpart2\bpart3\x64part4"
        logMessage = logMessage_prefix + logMessage_with_format_raw + "{i - 1}"
        [loop_breakpoint_id, post_loop_breakpoint_id] = self.set_source_breakpoints(
            self.main_path,
            [loop_line, after_loop_line],
            [{"logMessage": logMessage}, {}],
        )

        # Continue to trigger the breakpoint with log messages
        self.dap_server.request_continue()

        # Verify we hit the breakpoint after loop line
        self.verify_breakpoint_hit([post_loop_breakpoint_id])

        output = self.get_console()
        lines = output.splitlines()
        logMessage_output = []
        for line in lines:
            if line.startswith(logMessage_prefix):
                logMessage_output.append(line)

        # Verify logMessage count
        loop_count = 10
        self.assertEqual(len(logMessage_output), loop_count)

        # Verify log message match
        for idx, logMessage_line in enumerate(logMessage_output):
            result = idx - 1
            self.assertEqual(
                logMessage_line,
                logMessage_prefix + logMessage_with_format + str(result),
            )

    @skipIfWindows
    def test_logmessage_format_failure(self):
        """
        Tests breakpoint logmessage format with parsing failure.
        """
        before_loop_line = line_number("main.cpp", "// before loop")
        loop_line = line_number("main.cpp", "// break loop")
        after_loop_line = line_number("main.cpp", "// after loop")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set a breakpoint at a line before loop
        before_loop_breakpoint_ids = self.set_source_breakpoints(
            self.main_path, [before_loop_line]
        )
        self.assertEqual(len(before_loop_breakpoint_ids), 1, "expect one breakpoint")

        self.dap_server.request_continue()

        # Verify we hit the breakpoint before loop line
        self.verify_breakpoint_hit(before_loop_breakpoint_ids)

        # Swallow old console output
        self.get_console()

        # Set two breakpoints:
        # 1. First at the loop line with logMessage
        # 2. Second guard breakpoint at a line after loop
        logMessage_prefix = "This is log message for -- "
        # log message missing hex number.
        logMessage_with_format_raw = r"part1\x"
        logMessage = logMessage_prefix + logMessage_with_format_raw
        [loop_breakpoint_id, post_loop_breakpoint_id] = self.set_source_breakpoints(
            self.main_path,
            [loop_line, after_loop_line],
            [{"logMessage": logMessage}, {}],
        )

        # Continue to trigger the breakpoint with log messages
        self.dap_server.request_continue()

        # Verify we hit logpoint breakpoint if it's format has error.
        self.verify_breakpoint_hit([loop_breakpoint_id])

        output = self.get_console()
        lines = output.splitlines()

        failure_prefix = "Log message has error:"
        logMessage_output = []
        logMessage_failure_output = []
        for line in lines:
            if line.startswith(logMessage_prefix):
                logMessage_output.append(line)
            elif line.startswith(failure_prefix):
                logMessage_failure_output.append(line)

        # Verify logMessage failure message
        self.assertEqual(len(logMessage_failure_output), 1)
        self.assertEqual(
            logMessage_failure_output[0].strip(),
            failure_prefix + " missing hex number following '\\x'",
        )



# RUN: %PYTHON %s | FileCheck %s
# This is just a smoke test that the dialect is functional.

from mlir.ir import *
from mlir.dialects import ml_program, arith, builtin


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()
        print(module)
    return f


# CHECK-LABEL: testFuncOp
@constructAndPrintInModule
def testFuncOp():
    # CHECK: ml_program.func @foobar(%arg0: si32) -> si32
    f = ml_program.FuncOp(
        name="foobar", type=([IntegerType.get_signed(32)], [IntegerType.get_signed(32)])
    )
    block = f.add_entry_block()
    with InsertionPoint(block):
        # CHECK: ml_program.return
        ml_program.ReturnOp([block.arguments[0]])


# CHECK-LABEL: testGlobalStoreOp
@constructAndPrintInModule
def testGlobalStoreOp():
    # CHECK: %cst = arith.constant 4.242000e+01 : f32
    cst = arith.ConstantOp(value=42.42, result=F32Type.get())

    m = builtin.ModuleOp()
    m.sym_name = StringAttr.get("symbol1")
    m.sym_visibility = StringAttr.get("public")
    # CHECK: module @symbol1 attributes {sym_visibility = "public"} {
    # CHECK:   ml_program.global public mutable @symbol2 : f32
    # CHECK: }
    with InsertionPoint(m.body):
        ml_program.GlobalOp("symbol2", F32Type.get(), is_mutable=True)
    # CHECK: ml_program.global_store @symbol1::@symbol2 = %cst : f32
    ml_program.GlobalStoreOp(["symbol1", "symbol2"], cst)



"""
Test lldb-dap recieves invalidated-events when the area such as
stack, variables, threads has changes but the client does not
know about it.
"""

import lldbdap_testcase
from lldbsuite.test.lldbtest import line_number
from dap_server import Event


class TestDAP_invalidatedEvent(lldbdap_testcase.DAPTestCaseBase):
    def verify_top_frame_name(self, frame_name: str):
        all_frames = self.get_stackFrames()
        self.assertGreaterEqual(len(all_frames), 1, "Expected at least one frame.")
        top_frame_name = all_frames[0]["name"]
        self.assertRegex(top_frame_name, f"{frame_name}.*")

    def test_invalidated_stack_area_event(self):
        """
        Test an invalidated event for the stack area.
        The event is sent when the command `thread return <expr>` is sent by the user.
        """
        other_source = "other.h"
        return_bp_line = line_number(other_source, "// thread return breakpoint")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        self.set_source_breakpoints(other_source, [return_bp_line])
        self.continue_to_next_stop()

        self.verify_top_frame_name("add")
        thread_id = self.dap_server.get_thread_id()
        self.assertIsNotNone(thread_id, "Exepected a thread id.")

        # run thread return
        thread_command = "thread return 20"
        eval_resp = self.dap_server.request_evaluate(thread_command, context="repl")
        self.assertTrue(eval_resp["success"], f"Failed to evaluate `{thread_command}`.")

        # wait for the invalidated stack event.
        stack_event = self.dap_server.wait_for_event(["invalidated"])
        self.assertIsNotNone(stack_event, "Expected an invalidated event.")
        event_body: Event = stack_event["body"]
        self.assertIn("stacks", event_body["areas"])
        self.assertIn("threadId", event_body.keys())
        self.assertEqual(
            thread_id,
            event_body["threadId"],
            f"Expected the event from thread {thread_id}.",
        )

        # confirm we are back at the main frame.
        self.verify_top_frame_name("main")
        self.continue_to_exit()



# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

import os
import argparse

from jupyter_client.kernelspec import KernelSpecManager


def install_my_kernel_spec(user=True, prefix=None):
    """Install the kernel spec for user in given prefix."""
    print("Installing mlir-opt IPython kernel spec")
    pkgroot = os.path.dirname(__file__)
    KernelSpecManager().install_kernel_spec(
        os.path.join(pkgroot, "assets"), "mlir", user=user, prefix=prefix
    )


def _is_root():
    """Returns whether the current user is root."""
    try:
        return os.geteuid() == 0
    except AttributeError:
        # Return false wherever unknown.
        return False


def main(argv=None):
    parser = argparse.ArgumentParser(
        description="Install KernelSpec for MlirOpt Kernel"
    )
    prefix_locations = parser.add_mutually_exclusive_group()

    prefix_locations.add_argument(
        "--user", help="Install in user home directory", action="store_true"
    )
    prefix_locations.add_argument(
        "--prefix", help="Install directory prefix", default=None
    )

    args = parser.parse_args(argv)

    user = args.user or not _is_root()
    prefix = args.prefix

    install_my_kernel_spec(user=user, prefix=prefix)


if __name__ == "__main__":
    main()



"""
Test some SBStructuredData API.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

import json

class TestStructuredDataAPI(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.structured_data_api_test()

    def structured_data_api_test(self):
        error = lldb.SBError()
        s = lldb.SBStream()

        dict_str = json.dumps(
            {
                "key_dict": {
                    "key_string": "STRING",
                    "key_uint": 0xFFFFFFFF00000000,
                    "key_sint": -42,
                    "key_float": 2.99,
                    "key_bool": True,
                    "key_array": ["23", "arr"],
                }
            }
        )
        s.Print(dict_str)
        example = lldb.SBStructuredData()

        # Check SetFromJSON API for dictionaries, integers, floating point
        # values, strings and arrays
        error = example.SetFromJSON(s)
        if not error.Success():
            self.fail("FAILED:   " + error.GetCString())

        # Tests for invalid data type
        self.invalid_struct_test(example)

        # Test that GetDescription works:
        s.Clear()
        error = example.GetDescription(s)
        self.assertSuccess(error, "GetDescription works")
        # Ensure str() doesn't raise an exception.
        self.assertTrue(str(example))
        if not "key_float" in s.GetData():
            self.fail("FAILED: could not find key_float in description output")

        dict_struct = example.GetValueForKey("key_dict")

        # Tests for dictionary data type
        self.dictionary_struct_test(example)

        # Tests for string data type
        self.string_struct_test(dict_struct)

        # Tests for integer data type
        self.uint_struct_test(dict_struct)

        # Tests for integer data type
        self.sint_struct_test(dict_struct)

        # Tests for floating point data type
        self.double_struct_test(dict_struct)

        # Tests for boolean data type
        self.bool_struct_test(dict_struct)

        # Tests for array data type
        self.array_struct_test(dict_struct)

        s.Clear()
        self.assertSuccess(example.GetAsJSON(s))
        py_obj = json.loads(s.GetData())
        self.assertTrue(py_obj)
        self.assertIn("key_dict", py_obj)

        py_dict = py_obj["key_dict"]
        self.assertEqual(py_dict["key_string"], "STRING")
        self.assertEqual(py_dict["key_uint"], 0xFFFFFFFF00000000)
        self.assertEqual(py_dict["key_sint"], -42)
        self.assertEqual(py_dict["key_float"], 2.99)
        self.assertEqual(py_dict["key_bool"], True)
        self.assertEqual(py_dict["key_array"], ["23", "arr"])

        class MyRandomClass:
            payload = "foo"

        py_dict["key_generic"] = MyRandomClass()

        stp = lldb.SBScriptObject(py_dict, lldb.eScriptLanguagePython)
        self.assertEqual(stp.ptr, py_dict)

        sd = lldb.SBStructuredData(stp, self.dbg)
        self.assertTrue(sd.IsValid())
        self.assertEqual(sd.GetSize(), len(py_dict))

        generic_sd = sd.GetValueForKey("key_generic")
        self.assertTrue(generic_sd.IsValid())
        self.assertEqual(generic_sd.GetType(), lldb.eStructuredDataTypeGeneric)

        my_random_class = generic_sd.GetGenericValue()
        self.assertTrue(my_random_class)
        self.assertEqual(my_random_class.payload, MyRandomClass.payload)

        example = lldb.SBStructuredData()
        self.assertSuccess(example.SetFromJSON("1"))
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeInteger)
        self.assertEqual(example.GetIntegerValue(), 1)
        self.assertEqual(int(example), 1)

        self.assertSuccess(example.SetFromJSON("4.19"))
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeFloat)
        self.assertEqual(example.GetFloatValue(), 4.19)
        self.assertEqual(float(example), 4.19)

        self.assertSuccess(example.SetFromJSON('"Bonjour, 123!"'))
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeString)
        self.assertEqual(example.GetStringValue(42), "Bonjour, 123!")
        self.assertEqual(str(example), "Bonjour, 123!")

        self.assertSuccess(example.SetFromJSON("true"))
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeBoolean)
        self.assertTrue(example.GetBooleanValue())
        self.assertTrue(example)

        self.assertSuccess(example.SetFromJSON("null"))
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeNull)

        example = lldb.SBStructuredData()
        example.SetUnsignedIntegerValue(1)
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeInteger)
        self.assertEqual(example.GetIntegerValue(), 1)

        example.SetSignedIntegerValue(-42)
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeSignedInteger)
        self.assertEqual(example.GetSignedIntegerValue(), -42)

        example.SetFloatValue(4.19)
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeFloat)
        self.assertEqual(example.GetFloatValue(), 4.19)

        example.SetStringValue("Bonjour, 123!")
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeString)
        self.assertEqual(example.GetStringValue(42), "Bonjour, 123!")

        value = lldb.SBStructuredData()
        example.SetValueForKey("Hello", value)
        self.assertEqual(example.GetSize(), 0)

        nested_obj = lldb.SBStructuredData()
        nested_obj.SetStringValue("World")
        example.SetValueForKey("Hello", nested_obj)
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeDictionary)
        nested_obj = None
        nested_obj = example.GetValueForKey("Hello")
        self.assertTrue(nested_obj.IsValid())
        self.assertEqual(nested_obj.GetType(), lldb.eStructuredDataTypeString)
        self.assertEqual(nested_obj.GetStringValue(42), "World")

        example.SetBooleanValue(True)
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeBoolean)
        self.assertTrue(example.GetBooleanValue())

        rnd_obj = MyRandomClass()
        stp = lldb.SBScriptObject(rnd_obj, lldb.eScriptLanguagePython)
        self.assertEqual(stp.ptr, rnd_obj)

        example.SetGenericValue(stp)
        self.assertEqual(example.GetType(), lldb.eStructuredDataTypeGeneric)

        my_random_class = example.GetGenericValue()
        self.assertTrue(my_random_class)
        self.assertEqual(my_random_class.payload, MyRandomClass.payload)

        example_arr = [1, 2.3, "4", {"5": False}]
        arr_str = json.dumps(example_arr)
        s.Clear()
        s.Print(arr_str)
        self.assertSuccess(example.SetFromJSON(s))

        s.Clear()
        self.assertSuccess(example.GetAsJSON(s))
        sb_data = json.loads(s.GetData())
        self.assertEqual(sb_data, example_arr)

    def invalid_struct_test(self, example):
        invalid_struct = example.GetValueForKey("invalid_key")
        if invalid_struct.IsValid():
            self.fail("An invalid object should have been returned")

        # Check Type API
        if invalid_struct.GetType() != lldb.eStructuredDataTypeInvalid:
            self.fail("Wrong type returned: " + str(invalid_struct.GetType()))

    def dictionary_struct_test(self, example):
        # Check API returning a valid SBStructuredData of 'dictionary' type
        dict_struct = example.GetValueForKey("key_dict")
        if not dict_struct.IsValid():
            self.fail("A valid object should have been returned")

        # Check Type API
        if dict_struct.GetType() != lldb.eStructuredDataTypeDictionary:
            self.fail("Wrong type returned: " + str(dict_struct.GetType()))

        # Check Size API for 'dictionary' type
        if dict_struct.GetSize() != 6:
            self.fail("Wrong no of elements returned: " + str(dict_struct.GetSize()))

    def string_struct_test(self, dict_struct):
        string_struct = dict_struct.GetValueForKey("key_string")
        if not string_struct.IsValid():
            self.fail("A valid object should have been returned")

        # Check Type API
        if string_struct.GetType() != lldb.eStructuredDataTypeString:
            self.fail("Wrong type returned: " + str(string_struct.GetType()))

        # Check API returning 'string' value
        output = string_struct.GetStringValue(25)
        if not "STRING" in output:
            self.fail("wrong output: " + output)

        # Calling wrong API on a SBStructuredData
        # (e.g. getting an integer from a string type structure)
        output = string_struct.GetIntegerValue()
        if output:
            self.fail(
                "Valid integer value " + str(output) + " returned for a string object"
            )

    def uint_struct_test(self, dict_struct):
        # Check a valid SBStructuredData containing an unsigned integer.
        # We intentionally make this larger than what an int64_t can hold but
        # still small enough to fit a uint64_t
        uint_struct = dict_struct.GetValueForKey("key_uint")
        if not uint_struct.IsValid():
            self.fail("A valid object should have been returned")

        # Check Type API
        if uint_struct.GetType() != lldb.eStructuredDataTypeInteger:
            self.fail("Wrong type returned: " + str(uint_struct.GetType()))

        # Check API returning unsigned integer value
        output = uint_struct.GetUnsignedIntegerValue()
        if output != 0xFFFFFFFF00000000:
            self.fail("wrong output: " + str(output))

        # Calling wrong API on a SBStructuredData
        # (e.g. getting a string value from an integer type structure)
        output = uint_struct.GetStringValue(25)
        if output:
            self.fail("Valid string " + output + " returned for an integer object")

    def sint_struct_test(self, dict_struct):
        # Check a valid SBStructuredData containing an signed integer.
        # We intentionally make this smaller than what an uint64_t can hold but
        # still small enough to fit a int64_t
        sint_struct = dict_struct.GetValueForKey("key_sint")
        if not sint_struct.IsValid():
            self.fail("A valid object should have been returned")

        # Check Type API
        if sint_struct.GetType() != lldb.eStructuredDataTypeSignedInteger:
            self.fail("Wrong type returned: " + str(sint_struct.GetType()))

        # Check API returning signed integer value
        output = sint_struct.GetSignedIntegerValue()
        if output != -42:
            self.fail("wrong output: " + str(output))

        # Calling wrong API on a SBStructuredData
        # (e.g. getting a string value from an integer type structure)
        output = sint_struct.GetStringValue(69)
        if output:
            self.fail("Valid string " + output + " returned for an integer object")

    def double_struct_test(self, dict_struct):
        floating_point_struct = dict_struct.GetValueForKey("key_float")
        if not floating_point_struct.IsValid():
            self.fail("A valid object should have been returned")

        # Check Type API
        if floating_point_struct.GetType() != lldb.eStructuredDataTypeFloat:
            self.fail("Wrong type returned: " + str(floating_point_struct.GetType()))

        # Check API returning 'double' value
        output = floating_point_struct.GetFloatValue()
        if output != 2.99:
            self.fail("wrong output: " + str(output))

    def bool_struct_test(self, dict_struct):
        bool_struct = dict_struct.GetValueForKey("key_bool")
        if not bool_struct.IsValid():
            self.fail("A valid object should have been returned")

        # Check Type API
        if bool_struct.GetType() != lldb.eStructuredDataTypeBoolean:
            self.fail("Wrong type returned: " + str(bool_struct.GetType()))

        # Check API returning 'bool' value
        output = bool_struct.GetBooleanValue()
        if not output:
            self.fail("wrong output: " + str(output))

    def array_struct_test(self, dict_struct):
        # Check API returning a valid SBStructuredData of 'array' type
        array_struct = dict_struct.GetValueForKey("key_array")
        if not array_struct.IsValid():
            self.fail("A valid object should have been returned")

        # Check Type API
        if array_struct.GetType() != lldb.eStructuredDataTypeArray:
            self.fail("Wrong type returned: " + str(array_struct.GetType()))

        # Check Size API for 'array' type
        if array_struct.GetSize() != 2:
            self.fail("Wrong no of elements returned: " + str(array_struct.GetSize()))

        # Check API returning a valid SBStructuredData for different 'array'
        # indices
        string_struct = array_struct.GetItemAtIndex(0)
        if not string_struct.IsValid():
            self.fail("A valid object should have been returned")
        if string_struct.GetType() != lldb.eStructuredDataTypeString:
            self.fail("Wrong type returned: " + str(string_struct.GetType()))
        output = string_struct.GetStringValue(5)
        if output != "23":
            self.fail("wrong output: " + str(output))

        string_struct = array_struct.GetItemAtIndex(1)
        if not string_struct.IsValid():
            self.fail("A valid object should have been returned")
        if string_struct.GetType() != lldb.eStructuredDataTypeString:
            self.fail("Wrong type returned: " + str(string_struct.GetType()))
        output = string_struct.GetStringValue(5)
        if output != "arr":
            self.fail("wrong output: " + str(output))

    def test_round_trip_scalars(self):
        for original in (0, 11, -1, 0.0, 4.5, -0.25):
            constructor = type(original)
            data = lldb.SBStructuredData()
            data.SetFromJSON(json.dumps(original))
            round_tripped = constructor(data)
            self.assertEqual(round_tripped, original)

    def test_dynamic(self):
        for original in (0, 11, -1, 0.0, 4.5, -0.25, "", "dirk", True, False):
            data = lldb.SBStructuredData()
            data.SetFromJSON(json.dumps(original))
            self.assertEqual(data.dynamic, original)

    def test_round_trip_int(self):
        for original in (0, 11, -1):
            data = lldb.SBStructuredData()
            data.SetFromJSON(json.dumps(original))
            self.assertEqual(int(data), int(original))

    def test_round_trip_float(self):
        for original in (0, 11, -1, 0.0, 4.5, -0.25):
            data = lldb.SBStructuredData()
            data.SetFromJSON(json.dumps(original))
            self.assertEqual(float(data), float(original))

    def test_iterate_array(self):
        array = [0, 1, 2]
        data = lldb.SBStructuredData()
        data.SetFromJSON(json.dumps(array))
        for value in data:
            self.assertEqual(value, array.pop(0))

    def test_iterate_dictionary(self):
        dictionary = {"0": 0, "1": 1, "2": 2}
        keys = set(dictionary.keys())
        data = lldb.SBStructuredData()
        data.SetFromJSON(json.dumps(dictionary))
        for key in data:
            self.assertIn(key, keys)
            keys.remove(key)

    def test_getitem_array(self):
        array = [1, 2, 3]
        data = lldb.SBStructuredData()
        data.SetFromJSON(json.dumps(array))
        for i in range(len(array)):
            self.assertEqual(data[i], array[i])

    def test_getitem_dictionary(self):
        dictionary = {"one": 1, "two": 2, "three": 3}
        data = lldb.SBStructuredData()
        data.SetFromJSON(json.dumps(dictionary))
        for key in dictionary:
            self.assertEqual(data[key], dictionary[key])



"""
Make sure 'frame var' using DIL parser/evaultor works for local variables.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestFrameVarDILLocalVars(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_frame_var(self):
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")
        self.expect_var_path("a", value="1")
        self.expect_var_path("b", value="2")
        self.expect_var_path("c", value="'\\xfd'")
        self.expect_var_path("s", value="4")

        # Check that identifiers are allowed in both simple and legacy modes
        frame = thread.GetFrameAtIndex(0)
        simple = frame.GetValueForVariablePath("a", lldb.eDILModeSimple)
        legacy = frame.GetValueForVariablePath("a", lldb.eDILModeLegacy)
        self.assertSuccess(simple.GetError())
        self.assertSuccess(legacy.GetError())



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test_error(self):
        self.expect(
            "breakpoint set --func-regex (",
            error=True,
            substrs=[
                "error: Function name regular expression could "
                + "not be compiled: parentheses not balanced"
            ],
        )

        # Point out if looks like the user provided a globbing expression.
        self.expect(
            "breakpoint set --func-regex *a",
            error=True,
            substrs=[
                "error: Function name regular expression could "
                + "not be compiled: repetition-operator operand invalid",
                "warning: function name regex does not accept glob patterns",
            ],
        )
        self.expect(
            "breakpoint set --func-regex ?a",
            error=True,
            substrs=[
                "error: Function name regular expression could "
                + "not be compiled: repetition-operator operand invalid",
                "warning: function name regex does not accept glob patterns",
            ],
        )
        # Make sure that warning is only shown for invalid regular expressions
        # that look like a globbing expression (i.e., they have a leading * or ?).
        self.expect(
            "breakpoint set --func-regex a*+",
            error=True,
            matching=False,
            substrs=["warning: function name regex does not accept glob patterns"],
        )
        self.expect(
            "breakpoint set --func-regex a?+",
            error=True,
            matching=False,
            substrs=["warning: function name regex does not accept glob patterns"],
        )



import lldb
import json
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


def find(predicate, seq):
    for item in seq:
        if predicate(item):
            return item


class TestTraceSave(TraceIntelPTTestCaseBase):
    def testErrorMessages(self):
        # We first check the output when there are no targets
        self.expect(
            "trace save",
            substrs=[
                "error: invalid target, create a target using the 'target create' command"
            ],
            error=True,
        )

        # We now check the output when there's a non-running target
        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )

        self.expect(
            "trace save",
            substrs=["error: Command requires a current process."],
            error=True,
        )

        # Now we check the output when there's a running target without a trace
        self.expect("b main")
        self.expect("run")

        self.expect(
            "trace save", substrs=["error: Process is not being traced"], error=True
        )

    @skipIfNoIntelPT
    def testSaveToInvalidDir(self):
        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")
        self.expect("thread trace start")
        self.expect("n")

        # Check the output when saving without providing the directory argument
        self.expect(
            "trace save ",
            substrs=[
                "error: a single path to a directory where the trace bundle will be created is required"
            ],
            error=True,
        )

        # Check the output when saving to an invalid directory
        self.expect(
            "trace save /", substrs=["error: couldn't write to the file"], error=True
        )

    def testSaveWhenNotLiveTrace(self):
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json"),
            substrs=["intel-pt"],
        )

        # Check the output when not doing live tracing
        self.expect(
            "trace save "
            + os.path.join(self.getBuildDir(), "intelpt-trace", "trace_not_live_dir")
        )

    def testSaveMultiCpuTrace(self):
        """
        This test starts a per-cpu tracing session, then saves the session to disk, and
        finally it loads it again.
        """
        self.skipIfPerCpuTracingIsNotSupported()

        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")
        self.expect("process trace start --per-cpu-tracing")
        self.expect("b 7")

        output_dir = os.path.join(self.getBuildDir(), "intelpt-trace", "trace_save")
        self.expect("trace save " + output_dir)

        def checkSessionBundle(session_file_path):
            with open(session_file_path) as session_file:
                session = json.load(session_file)
                # We expect tsc conversion info
                self.assertIn("tscPerfZeroConversion", session)
                # We expect at least one cpu
                self.assertGreater(len(session["cpus"]), 0)

                # We expect the required trace files to be created
                for cpu in session["cpus"]:
                    cpu_files_prefix = os.path.join(output_dir, "cpus", str(cpu["id"]))
                    self.assertTrue(os.path.exists(cpu_files_prefix + ".intelpt_trace"))
                    self.assertTrue(
                        os.path.exists(cpu_files_prefix + ".perf_context_switch_trace")
                    )

                # We expect at least one one process
                self.assertGreater(len(session["processes"]), 0)
                for process in session["processes"]:
                    # We expect at least one thread
                    self.assertGreater(len(process["threads"]), 0)
                    # We don't expect thread traces
                    for thread in process["threads"]:
                        self.assertTrue(
                            ("iptTrace" not in thread) or (thread["iptTrace"] is None)
                        )

        original_trace_session_file = os.path.join(output_dir, "trace.json")
        checkSessionBundle(original_trace_session_file)

        output_dir = os.path.join(self.getBuildDir(), "intelpt-trace", "trace_save")
        self.expect("trace load " + os.path.join(output_dir, "trace.json"))
        output_copy_dir = os.path.join(
            self.getBuildDir(), "intelpt-trace", "copy_trace_save"
        )
        self.expect("trace save " + output_copy_dir)

        # We now check that the new bundle is correct on its own
        copied_trace_session_file = os.path.join(output_copy_dir, "trace.json")
        checkSessionBundle(copied_trace_session_file)

        # We finally check that the new bundle has the same information as the original one
        with open(original_trace_session_file) as original_file:
            original = json.load(original_file)
            with open(copied_trace_session_file) as copy_file:
                copy = json.load(copy_file)

                self.assertEqual(len(original["processes"]), len(copy["processes"]))

                for process in original["processes"]:
                    copied_process = find(
                        lambda proc: proc["pid"] == process["pid"], copy["processes"]
                    )
                    self.assertIsNotNone(copied_process)

                    for thread in process["threads"]:
                        copied_thread = find(
                            lambda thr: thr["tid"] == thread["tid"],
                            copied_process["threads"],
                        )
                        self.assertIsNotNone(copied_thread)

                for cpu in original["cpus"]:
                    copied_cpu = find(lambda cor: cor["id"] == cpu["id"], copy["cpus"])
                    self.assertIsNotNone(copied_cpu)

    @skipIfNoIntelPT
    def testSaveTrace(self):
        self.expect(
            "target create "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")
        self.expect("thread trace start")
        self.expect("b 7")

        ci = self.dbg.GetCommandInterpreter()
        res = lldb.SBCommandReturnObject()

        ci.HandleCommand("thread trace dump instructions -c 10 --forwards", res)
        self.assertTrue(res.Succeeded())
        first_ten_instructions = res.GetOutput()

        ci.HandleCommand("thread trace dump instructions -c 10", res)
        self.assertTrue(res.Succeeded())
        last_ten_instructions = res.GetOutput()

        # Now, save the trace to <trace_copy_dir>
        self.expect(
            "trace save "
            + os.path.join(self.getBuildDir(), "intelpt-trace", "trace_copy_dir")
        )

        # Load the trace just saved
        self.expect(
            "trace load -v "
            + os.path.join(
                self.getBuildDir(), "intelpt-trace", "trace_copy_dir", "trace.json"
            ),
            substrs=["intel-pt"],
        )

        # Compare with instructions saved at the first time
        ci.HandleCommand("thread trace dump instructions -c 10 --forwards", res)
        self.assertTrue(res.Succeeded())
        self.assertEqual(res.GetOutput(), first_ten_instructions)

        ci.HandleCommand("thread trace dump instructions -c 10", res)
        self.assertTrue(res.Succeeded())
        self.assertEqual(res.GetOutput(), last_ten_instructions)

    def testSaveKernelTrace(self):
        original_trace_file = os.path.join(
            self.getSourceDir(), "intelpt-kernel-trace", "trace.json"
        )
        copied_trace_dir = os.path.join(self.getBuildDir(), "intelpt-kernel-trace")
        copied_trace_file = os.path.join(copied_trace_dir, "trace.json")

        self.expect("trace load -v " + original_trace_file, substrs=["intel-pt"])
        self.expect("trace save " + copied_trace_dir)

        # We finally check that the new json has the same information as the original one
        with open(original_trace_file) as original_file:
            original_file = json.load(original_file)
            with open(copied_trace_file) as copy_file:
                copy_file = json.load(copy_file)
                self.assertIn("kernel", copy_file)

                self.assertEqual(
                    os.path.basename(original_file["kernel"]["file"]),
                    os.path.basename(copy_file["kernel"]["file"]),
                )



"""
Test lldb 'image list' on object files across multiple architectures.
This exercises classes like ObjectFileELF and their support for opening
foreign-architecture object files.
"""


import os.path
import re

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestImageListMultiArchitecture(TestBase):
    @no_debug_info_test
    @skipIfRemote
    def test_image_list_shows_multiple_architectures(self):
        """Test that image list properly shows the correct architecture for a set of different architecture object files."""
        images = {
            "hello-freebsd-10.0-x86_64-clang-3.3": re.compile(
                r"x86_64-(\*)?-freebsd10.0(-unknown)? x86_64"
            ),
            "hello-freebsd-10.0-x86_64-gcc-4.7.3": re.compile(
                r"x86_64-(\*)?-freebsd10.0(-unknown)? x86_64"
            ),
            "hello-netbsd-6.1-x86_64-gcc-4.5.3": re.compile(
                r"x86_64-(\*)?-netbsd6.1.4(-unknown)? x86_64"
            ),
            "hello-ubuntu-14.04-x86_64-gcc-4.8.2": re.compile(
                r"x86_64-(\*)?-linux(-unknown)? x86_64"
            ),
            "hello-ubuntu-14.04-x86_64-clang-3.5pre": re.compile(
                r"x86_64-(\*)?-linux(-unknown)? x86_64"
            ),
        }

        for image_name in images:
            file_name = os.path.abspath(
                os.path.join(os.path.dirname(__file__), "bin", image_name)
            )
            expected_triple_and_arch_regex = images[image_name]

            self.runCmd("file {}".format(file_name))
            self.match("image list -t -A", [expected_triple_and_arch_regex])
        # Revert to the host platform after all of this is done
        self.runCmd("platform select host")



"""
Test that we stop at runtime instrumentation locations (asan).
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbdap_testcase


class TestDAP_asan(lldbdap_testcase.DAPTestCaseBase):
    @skipUnlessAddressSanitizer
    def test_asan(self):
        """
        Test that we stop at asan.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        self.do_continue()

        self.verify_stop_exception_info("Use of deallocated memory")
        exceptionInfo = self.get_exceptionInfo()
        self.assertEqual(exceptionInfo["breakMode"], "always")
        self.assertRegex(
            exceptionInfo["description"], r"fatal_error: heap-use-after-free"
        )
        self.assertEqual(exceptionInfo["exceptionId"], "runtime-instrumentation")



import os
import shutil
import tempfile

import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


"""
Test support for the DebugInfoD network symbol acquisition protocol.
This one is for simple / no split-dwarf scenarios.

For no-split-dwarf scenarios, there are 2 variations:
1 - A stripped binary with it's corresponding unstripped binary:
2 - A stripped binary with a corresponding --only-keep-debug symbols file
"""


class DebugInfodTests(TestBase):
    # No need to try every flavor of debug inf.
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    @skipUnlessPlatform(["linux", "freebsd"])
    def test_normal_no_symbols(self):
        """
        Validate behavior with no symbols or symbol locator.
        ('baseline negative' behavior)
        """
        test_root = self.config_test(["a.out"])
        self.try_breakpoint(False)

    @skipUnlessPlatform(["linux", "freebsd"])
    def test_normal_default(self):
        """
        Validate behavior with symbols, but no symbol locator.
        ('baseline positive' behavior)
        """
        test_root = self.config_test(["a.out", "a.out.debug"])
        self.try_breakpoint(True)

    @skipIfCurlSupportMissing
    @skipUnlessPlatform(["linux", "freebsd"])
    def test_debuginfod_symbols(self):
        """
        Test behavior with the full binary available from Debuginfod as
        'debuginfo' from the plug-in.
        """
        test_root = self.config_test(["a.out"], "a.out.unstripped")
        self.try_breakpoint(True)

    @skipIfCurlSupportMissing
    @skipUnlessPlatform(["linux", "freebsd"])
    def test_debuginfod_executable(self):
        """
        Test behavior with the full binary available from Debuginfod as
        'executable' from the plug-in.
        """
        test_root = self.config_test(["a.out"], None, "a.out.unstripped")
        self.try_breakpoint(True)

    @skipIfCurlSupportMissing
    @skipUnlessPlatform(["linux", "freebsd"])
    def test_debuginfod_okd_symbols(self):
        """
        Test behavior with the 'only-keep-debug' symbols available from Debuginfod.
        """
        test_root = self.config_test(["a.out"], "a.out.debug")
        self.try_breakpoint(True)

    def try_breakpoint(self, should_have_loc):
        """
        This function creates a target from self.aout, sets a function-name
        breakpoint, and checks to see if we have a file/line location,
        as a way to validate that the symbols have been loaded.
        should_have_loc specifies if we're testing that symbols have or
        haven't been loaded.
        """
        target = self.dbg.CreateTarget(self.aout)
        self.assertTrue(target and target.IsValid(), "Target is valid")

        bp = target.BreakpointCreateByName("func")
        self.assertTrue(bp and bp.IsValid(), "Breakpoint is valid")
        self.assertEqual(bp.GetNumLocations(), 1)

        loc = bp.GetLocationAtIndex(0)
        self.assertTrue(loc and loc.IsValid(), "Location is valid")
        addr = loc.GetAddress()
        self.assertTrue(addr and addr.IsValid(), "Loc address is valid")
        line_entry = addr.GetLineEntry()
        self.assertEqual(
            should_have_loc,
            line_entry != None and line_entry.IsValid(),
            "Loc line entry is valid",
        )
        if should_have_loc:
            self.assertEqual(line_entry.GetLine(), 4)
            self.assertEqual(
                line_entry.GetFileSpec().GetFilename(),
                self.main_source_file.GetFilename(),
            )
        self.dbg.DeleteTarget(target)
        shutil.rmtree(self.tmp_dir)

    def config_test(self, local_files, debuginfo=None, executable=None):
        """
        Set up a test with local_files[] copied to a different location
        so that we control which files are, or are not, found in the file system.
        Also, create a stand-alone file-system 'hosted' debuginfod server with the
        provided debuginfo and executable files (if they exist)

        Make the filesystem look like:

        /tmp/<tmpdir>/test/[local_files]

        /tmp/<tmpdir>/cache (for lldb to use as a temp cache)

        /tmp/<tmpdir>/buildid/<uuid>/executable -> <executable>
        /tmp/<tmpdir>/buildid/<uuid>/debuginfo -> <debuginfo>
        Returns the /tmp/<tmpdir> path
        """

        self.build()

        uuid = self.getUUID("a.out")
        if not uuid:
            self.fail("Could not get UUID for a.out")
            return
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.tmp_dir = tempfile.mkdtemp()
        test_dir = os.path.join(self.tmp_dir, "test")
        os.makedirs(test_dir)

        self.aout = ""
        # Copy the files used by the test:
        for f in local_files:
            shutil.copy(self.getBuildArtifact(f), test_dir)
            # The first item is the binary to be used for the test
            if self.aout == "":
                self.aout = os.path.join(test_dir, f)

        use_debuginfod = debuginfo != None or executable != None

        # Populated the 'file://... mocked' Debuginfod server:
        if use_debuginfod:
            os.makedirs(os.path.join(self.tmp_dir, "cache"))
            uuid_dir = os.path.join(self.tmp_dir, "buildid", uuid)
            os.makedirs(uuid_dir)
            if debuginfo:
                shutil.copy(
                    self.getBuildArtifact(debuginfo),
                    os.path.join(uuid_dir, "debuginfo"),
                )
            if executable:
                shutil.copy(
                    self.getBuildArtifact(executable),
                    os.path.join(uuid_dir, "executable"),
                )

        # Configure LLDB for the test:
        self.runCmd(
            "settings set symbols.enable-external-lookup %s"
            % str(use_debuginfod).lower()
        )
        self.runCmd("settings clear plugin.symbol-locator.debuginfod.server-urls")
        if use_debuginfod:
            self.runCmd(
                "settings set plugin.symbol-locator.debuginfod.cache-path %s/cache"
                % self.tmp_dir
            )
            self.runCmd(
                "settings insert-before plugin.symbol-locator.debuginfod.server-urls 0 file://%s"
                % self.tmp_dir
            )

    def getUUID(self, filename):
        try:
            spec = lldb.SBModuleSpec()
            spec.SetFileSpec(lldb.SBFileSpec(self.getBuildArtifact(filename)))
            module = lldb.SBModule(spec)
            uuid = module.GetUUIDString().replace("-", "").lower()
            # Don't want lldb's fake 32 bit CRC's for this one
            return uuid if len(uuid) > 8 else None
        except:
            return None



"""
Test that the FrameRecognizer for __abort_with_payload
works properly
"""


import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestAbortWithPayload(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessAppleSilicon
    def test_abort_with_payload(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.abort_with_test(True)

    @skipUnlessAppleSilicon
    def test_abort_with_reason(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.abort_with_test(False)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.main_source_file = lldb.SBFileSpec("main.c")

    def abort_with_test(self, with_payload):
        """If with_payload is True, we test the abort_with_payload call,
        if false, we test abort_with_reason."""
        launch_info = lldb.SBLaunchInfo([])
        if not with_payload:
            launch_info.SetArguments(["use_reason"], True)
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self,
            "Stop here before abort",
            self.main_source_file,
            launch_info=launch_info,
        )

        frame = thread.GetFrameAtIndex(0)
        payload_str_var = frame.FindVariable("payload_string")
        self.assertSuccess(payload_str_var.GetError(), "Got payload string var")
        payload_var_addr = payload_str_var.unsigned

        payload_size_var = frame.FindVariable("payload_string_len")
        self.assertSuccess(payload_size_var.GetError(), "Got payload string len var")
        payload_size_val = payload_size_var.unsigned

        # Not let it run to crash:
        process.Continue()

        # At this point we should have stopped at the internal function.
        # Make sure we selected the right thread:
        sel_thread = process.GetSelectedThread()
        self.assertEqual(thread, sel_thread, "Selected the original thread")
        # Make sure the stop reason is right:
        self.assertEqual(
            thread.stop_description,
            "abort with payload or reason",
            "Description was right",
        )
        frame_0 = thread.frames[0]
        self.assertEqual(frame_0.name, "__abort_with_payload", "Frame 0 was right")

        # Now check the recognized argument values and the ExtendedCrashInformation version:
        options = lldb.SBVariablesOptions()
        options.SetIncludeRecognizedArguments(True)
        options.SetIncludeArguments(False)
        options.SetIncludeLocals(False)
        options.SetIncludeStatics(False)
        options.SetIncludeRuntimeSupportValues(False)

        arguments = frame_0.GetVariables(options)

        correct_values = {
            "namespace": 5,
            "code": 100,
            "payload_addr": payload_var_addr,
            "payload_size": payload_size_val,
            "payload_string": '"This is a payload that happens to be a string"',
            "reason_string": '"This is the reason string"',
            "reason_no_quote": "This is the reason string",
            "flags": 0x85,
        }

        # First check the recognized argument values:
        self.assertEqual(len(arguments), 6, "Got all six values")
        self.assertEqual(arguments[0].name, "namespace")
        self.assertEqual(
            arguments[0].unsigned,
            correct_values["namespace"],
            "Namespace value correct",
        )

        self.assertEqual(arguments[1].name, "code")
        self.assertEqual(
            arguments[1].unsigned, correct_values["code"], "code value correct"
        )

        # We always stop at __abort_with_payload, regardless of whether the caller
        # was abort_with_reason or abort_with_payload or any future API that
        # funnels here. Since I don't want to have to know too much about the
        # callers, I just always report what is in the function I've 
        # 
        # add the payload ones if it is the payload not the reason function.
        self.assertEqual(arguments[2].name, "payload_addr")
        self.assertEqual(arguments[3].name, "payload_size")
        if with_payload:
            self.assertEqual(
                arguments[2].unsigned,
                correct_values["payload_addr"],
                "Payload matched variable address",
            )
            # We've made a payload that is a string, try to fetch that:
            char_ptr_type = target.FindFirstType("char").GetPointerType()
            self.assertTrue(char_ptr_type.IsValid(), "Got char ptr type")

            str_val = arguments[2].Cast(char_ptr_type)
            self.assertEqual(
                str_val.summary, correct_values["payload_string"], "Got payload string"
            )

            self.assertEqual(
                arguments[3].unsigned,
                correct_values["payload_size"],
                "payload size value correct",
            )
        else:
            self.assertEqual(
                arguments[2].unsigned, 0, "Got 0 payload addr for reason call"
            )
            self.assertEqual(
                arguments[3].unsigned, 0, "Got 0 payload size for reason call"
            )

        self.assertEqual(arguments[4].name, "reason")
        self.assertEqual(
            arguments[4].summary,
            correct_values["reason_string"],
            "Reason value correct",
        )

        self.assertEqual(arguments[5].name, "flags")
        self.assertEqual(
            arguments[5].unsigned, correct_values["flags"], "Flags value correct"
        )

        # Also check that the same info was stored in the ExtendedCrashInformation dict:
        dict = process.GetExtendedCrashInformation()
        self.assertTrue(dict.IsValid(), "Got extended crash information dict")
        self.assertEqual(
            dict.GetType(), lldb.eStructuredDataTypeDictionary, "It is a dictionary"
        )

        abort_dict = dict.GetValueForKey("abort_with_payload")
        self.assertTrue(abort_dict.IsValid(), "Got an abort_with_payload dict")
        self.assertEqual(
            abort_dict.GetType(),
            lldb.eStructuredDataTypeDictionary,
            "It is a dictionary",
        )

        namespace_val = abort_dict.GetValueForKey("namespace")
        self.assertTrue(namespace_val.IsValid(), "Got a valid namespace")
        self.assertEqual(
            namespace_val.GetIntegerValue(0),
            correct_values["namespace"],
            "Namespace value correct",
        )

        code_val = abort_dict.GetValueForKey("code")
        self.assertTrue(code_val.IsValid(), "Got a valid code")
        self.assertEqual(
            code_val.GetIntegerValue(0), correct_values["code"], "Code value correct"
        )

        if with_payload:
            addr_val = abort_dict.GetValueForKey("payload_addr")
            self.assertTrue(addr_val.IsValid(), "Got a payload_addr")
            self.assertEqual(
                addr_val.GetIntegerValue(0),
                correct_values["payload_addr"],
                "payload_addr right in dictionary",
            )

            size_val = abort_dict.GetValueForKey("payload_size")
            self.assertTrue(size_val.IsValid(), "Got a payload size value")
            self.assertEqual(
                size_val.GetIntegerValue(0),
                correct_values["payload_size"],
                "payload size right in dictionary",
            )

        reason_val = abort_dict.GetValueForKey("reason")
        self.assertTrue(reason_val.IsValid(), "Got a reason key")
        self.assertEqual(
            reason_val.GetStringValue(100),
            correct_values["reason_no_quote"],
            "reason right in dictionary",
        )

        flags_val = abort_dict.GetValueForKey("flags")
        self.assertTrue(flags_val.IsValid(), "Got a flags value")
        self.assertEqual(
            flags_val.GetIntegerValue(0),
            correct_values["flags"],
            "flags right in dictionary",
        )



# RUN: %PYTHON %s | FileCheck %s
# This is just a smoke test that the dialect is functional.

from mlir.ir import *
from mlir.dialects import amdgpu, func


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()
        print(module)
    return f


# CHECK-LABEL: testSmoke
@constructAndPrintInModule
def testSmoke():
    # CHECK: amdgpu.lds_barrier
    amdgpu.LDSBarrierOp()


# CHECK-LABEL: testFatRawBufferCastOpParams
@constructAndPrintInModule
def testFatRawBufferCastOpParams():
    memref_type = MemRefType.get(
        [ShapedType.get_dynamic_size(), ShapedType.get_dynamic_size()],
        F32Type.get(),
    )
    f = func.FuncOp("test_raw_buffer_cast_params", ([memref_type], []))
    with InsertionPoint(f.add_entry_block()):
        block_args = f.arguments
        amdgpu.FatRawBufferCastOp(block_args[0])
        amdgpu.FatRawBufferCastOp(block_args[0], resetOffset=True)
        amdgpu.FatRawBufferCastOp(block_args[0], boundsCheck=False)
        amdgpu.FatRawBufferCastOp(block_args[0], boundsCheck=False, resetOffset=True)
        func.ReturnOp([])

    # CHECK:     func.func @test_raw_buffer_cast_params(%[[ARG0:.+]]: memref<?x?xf32>) {
    # CHECK:        amdgpu.fat_raw_buffer_cast %[[ARG0]] : memref<?x?xf32> to memref<?x?xf32, #amdgpu.address_space<fat_raw_buffer>>
    # CHECK-NEXT:   amdgpu.fat_raw_buffer_cast %[[ARG0]] resetOffset : memref<?x?xf32> to memref<?x?xf32, #amdgpu.address_space<fat_raw_buffer>>
    # CHECK-NEXT:   amdgpu.fat_raw_buffer_cast %[[ARG0]] boundsCheck(false) : memref<?x?xf32> to memref<?x?xf32, #amdgpu.address_space<fat_raw_buffer>>
    # CHECK-NEXT:   amdgpu.fat_raw_buffer_cast %[[ARG0]] boundsCheck(false) resetOffset : memref<?x?xf32> to memref<?x?xf32, #amdgpu.address_space<fat_raw_buffer>>


# CHECK-LABEL: testTDMTypes
@run
def testTDMTypes():
    with Context():
        f32 = F32Type.get()
        i32 = IntegerType.get_signless(32)

        # CHECK: !amdgpu.tdm_base<f32>
        tdm_base = amdgpu.TDMBaseType.get(f32)
        print(tdm_base)

        # CHECK: !amdgpu.tdm_descriptor
        tdm_descriptor = amdgpu.TDMDescriptorType.get()
        print(tdm_descriptor)

        # CHECK: !amdgpu.tdm_gather_base<f32, i32>
        tdm_gather_base = amdgpu.TDMGatherBaseType.get(f32, i32)
        print(tdm_gather_base)



"""Test that importing modules in Objective-C works as expected."""


import lldb

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCModulesAutoImportTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.m", "// Set breakpoint 0 here.")

    @skipIf(macos_version=["<", "10.12"])
    @skipIf(compiler="clang", compiler_version=["<", "19.0"])
    def test_expr(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break inside the foo function which takes a bar_ptr argument.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        self.runCmd("settings set target.auto-import-clang-modules true")

        self.expect(
            "expression getpid()", VARIABLES_DISPLAYED_CORRECTLY, substrs=["pid_t"]
        )



"""
Test lldb-dap output events
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbdap_testcase


class TestDAP_output(lldbdap_testcase.DAPTestCaseBase):
    @skipIfWindows
    def test_output(self):
        """
        Test output handling for the running process.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(
            program,
            disconnectAutomatically=False,
            exitCommands=[
                # Ensure that output produced by lldb itself is not consumed by the OutputRedirector.
                "?script print('out\\0\\0', end='\\r\\n', file=sys.stdout)",
                "?script print('err\\0\\0', end='\\r\\n', file=sys.stderr)",
            ],
        )
        source = "main.c"
        lines = [line_number(source, "// breakpoint 1")]
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.continue_to_breakpoints(breakpoint_ids)

        # Ensure partial messages are still sent.
        output = self.collect_stdout(pattern="abcdef")
        self.assertTrue(output and len(output) > 0, "expect program stdout")

        self.continue_to_exit()

        # Disconnecting from the server to ensure any pending IO is flushed.
        self.dap_server.request_disconnect()

        output += self.get_stdout()
        self.assertTrue(output and len(output) > 0, "expect program stdout")
        self.assertIn(
            "abcdefghi\r\nhello world\r\nfinally\0\0",
            output,
            "full stdout not found in: " + repr(output),
        )
        console = self.get_console()
        self.assertTrue(console and len(console) > 0, "expect dap messages")
        self.assertIn(
            "out\0\0\r\nerr\0\0\r\n", console, f"full console message not found"
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class TestVLA(TestBase):
    @skipIf(compiler="clang", compiler_version=["<", "8.0"])
    def test_variable_list(self):
        self.build()
        _, process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )

        # Make sure no helper expressions show up in frame variable.
        var_opts = lldb.SBVariablesOptions()
        var_opts.SetIncludeArguments(False)
        var_opts.SetIncludeLocals(True)
        var_opts.SetInScopeOnly(True)
        var_opts.SetIncludeStatics(False)
        var_opts.SetIncludeRuntimeSupportValues(False)
        var_opts.SetUseDynamic(lldb.eDynamicCanRunTarget)
        all_locals = self.frame().GetVariables(var_opts)
        for value in all_locals:
            self.assertNotIn("vla_expr", value.name)

    @decorators.skipIf(compiler="clang", compiler_version=["<", "8.0"])
    def test_vla(self):
        self.build()
        _, process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )

        def test(a):
            children = []
            for i in range(a):
                name = "[%d]" % i
                value = str(a - i)
                self.expect_var_path("vla" + name, type="int", value=value)
                self.expect_expr("vla" + name, result_type="int", result_value=value)
                children.append(ValueCheck(name=name, value=value))
            self.expect_var_path("vla", type="int[]", children=children)
            self.expect("expr vla", error=True, substrs=["incomplete"])

        test(2)
        process.Continue()
        test(4)



"""
Test code for dereferencing synthetic wrapped pointers.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DILSyntheticDereferenceTestCase(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_frame_var(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")
        self.runCmd("script from wrapPtrSynthProvider import *")
        self.runCmd("type synth add -l wrapPtrSynthProvider wrap_ptr")

        self.expect_var_path("ptr_node->value", value="1")
        self.expect_var_path("ptr_node->next->value", value="2")
        self.expect_var_path("(*ptr_node).value", value="1")
        self.expect_var_path("(*(*ptr_node).next).value", value="2")

        self.expect_var_path("ptr_node.ptr", type="NodeS *")
        self.expect_var_path("ptr_node.ptr->value", value="1")
        self.expect_var_path("ptr_node.ptr->next.ptr->value", value="2")



import lldb


class FooSyntheticProvider:
    def __init__(self, valobj, dict):
        target = valobj.GetTarget()
        data = lldb.SBData.CreateDataFromCString(lldb.eByteOrderLittle, 8, "S")
        self._child = valobj.CreateValueFromData(
            "synth_child", data, target.GetBasicType(lldb.eBasicTypeChar)
        )

    def num_children(self):
        return 1

    def get_child_at_index(self, index):
        if index != 0:
            return None
        return self._child

    def get_child_index(self, name):
        if name == "synth_child":
            return 0
        return None



import lldb


num_children_calls = []


class TestSyntheticProvider:
    def __init__(self, valobj, dict):
        target = valobj.GetTarget()
        self._type = valobj.GetType()
        data = lldb.SBData.CreateDataFromCString(lldb.eByteOrderLittle, 8, "S")
        name = "child" if "Not" in self._type.GetName() else "[0]"
        self._child = valobj.CreateValueFromData(
            name, data, target.GetBasicType(lldb.eBasicTypeChar)
        )

    def num_children(self):
        num_children_calls.append(self._type.GetName())
        return 1

    def get_child_at_index(self, index):
        if index != 0:
            return None
        return self._child

    def get_child_index(self, name):
        if name == self._child.GetName():
            return 0
        return None


def __lldb_init_module(debugger, dict):
    cat = debugger.CreateCategory("TestCategory")
    cat.AddTypeSynthetic(
        lldb.SBTypeNameSpecifier("Indexed"),
        lldb.SBTypeSynthetic.CreateWithClassName("formatter.TestSyntheticProvider"),
    )
    cat.AddTypeSynthetic(
        lldb.SBTypeNameSpecifier("NotIndexed"),
        lldb.SBTypeSynthetic.CreateWithClassName("formatter.TestSyntheticProvider"),
    )
    cat.SetEnabled(True)



"""
Provides definitions for various lldb test categories
"""

# System modules
import sys

# Third-party modules

# LLDB modules
from lldbsuite.support import gmodules

# Key: Category name
# Value: should be used in lldbtest's debug-info replication
debug_info_categories = {
    "dwarf": True,
    "dwo": True,
    "dsym": True,
    "pdb": False,
    "gmodules": False,
}

all_categories = {
    "basic_process": "Basic process execution sniff tests.",
    "cmdline": "Tests related to the LLDB command-line interface",
    "dataformatters": "Tests related to the type command and the data formatters subsystem",
    "debugserver": "Debugserver tests",
    "dsym": "Tests that can be run with DSYM debug information",
    "dwarf": "Tests that can be run with DWARF debug information",
    "dwo": "Tests that can be run with DWO debug information",
    "dyntype": "Tests related to dynamic type support",
    "expression": "Tests related to the expression parser",
    "flakey": "Flakey test cases, i.e. tests that do not reliably pass at each execution",
    "fork": "Tests requiring the process plugin fork/vfork event support",
    "gmodules": "Tests that can be run with -gmodules debug information",
    "instrumentation-runtime": "Tests for the instrumentation runtime plugins",
    "libc++": "Test for libc++ data formatters",
    "libstdcxx": "Test for libstdcxx data formatters",
    "lldb-server": "Tests related to lldb-server",
    "lldb-dap": "Tests for the Debug Adapter Protocol with lldb-dap",
    "llgs": "Tests for the gdb-server functionality of lldb-server",
    "msvcstl": "Test for MSVC STL data formatters",
    "pdb": "Tests that can be run with PDB debug information",
    "pexpect": "Tests requiring the pexpect library to be available",
    "objc": "Tests related to the Objective-C programming language support",
    "pyapi": "Tests related to the Python API",
    "std-module": "Tests related to importing the std module",
    "stresstest": "Tests related to stressing lldb limits",
    "watchpoint": "Watchpoint-related tests",
}


def unique_string_match(yourentry, list):
    candidate = None
    for item in list:
        if not item.startswith(yourentry):
            continue
        if candidate:
            return None
        candidate = item
    return candidate


def is_supported_on_platform(category, platform, compiler_path):
    if category == "dwo":
        # -gsplit-dwarf is not implemented by clang on Windows.
        return platform in ["linux", "freebsd"]
    elif category == "dsym":
        return platform in ["darwin", "macosx", "ios", "watchos", "tvos", "bridgeos"]
    elif category == "gmodules":
        # First, check to see if the platform can even support gmodules.
        if platform not in ["darwin", "macosx", "ios", "watchos", "tvos", "bridgeos"]:
            return False
        return gmodules.is_compiler_clang_with_gmodules(compiler_path)
    elif category == "pdb":
        return platform == "windows"
    return True


def validate(categories, exact_match):
    """
    For each category in categories, ensure that it's a valid category (if exact_match is false,
    unique prefixes are also accepted). If a category is invalid, print a message and quit.
       If all categories are valid, return the list of categories. Prefixes are expanded in the
       returned list.
    """
    result = []
    for category in categories:
        origCategory = category
        if category not in all_categories and not exact_match:
            category = unique_string_match(category, all_categories)
        if (category not in all_categories) or category is None:
            print(
                "fatal error: category '" + origCategory + "' is not a valid category"
            )
            print(
                "if you have added a new category, please edit test_categories.py, adding your new category to all_categories"
            )
            print(
                "else, please specify one or more of the following: "
                + str(list(all_categories.keys()))
            )
            sys.exit(1)
        result.append(category)
    return result



"""
Test LLDB type promotion of unscoped enums.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCPPEnumPromotion(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )
        UChar_promoted = self.frame().FindVariable("UChar_promoted")
        UShort_promoted = self.frame().FindVariable("UShort_promoted")
        UInt_promoted = self.frame().FindVariable("UInt_promoted")
        SLong_promoted = self.frame().FindVariable("SLong_promoted")
        ULong_promoted = self.frame().FindVariable("ULong_promoted")
        NChar_promoted = self.frame().FindVariable("NChar_promoted")
        NShort_promoted = self.frame().FindVariable("NShort_promoted")
        NInt_promoted = self.frame().FindVariable("NInt_promoted")
        NLong_promoted = self.frame().FindVariable("NLong_promoted")

        # Check that LLDB's promoted type is the same as the compiler's
        self.expect_expr("+EnumUChar::UChar", result_type=UChar_promoted.type.name)
        self.expect_expr("+EnumUShort::UShort", result_type=UShort_promoted.type.name)
        self.expect_expr("+EnumUInt::UInt", result_type=UInt_promoted.type.name)
        self.expect_expr("+EnumSLong::SLong", result_type=SLong_promoted.type.name)
        self.expect_expr("+EnumULong::ULong", result_type=ULong_promoted.type.name)
        self.expect_expr("+EnumNChar::NChar", result_type=NChar_promoted.type.name)
        self.expect_expr("+EnumNShort::NShort", result_type=NShort_promoted.type.name)
        self.expect_expr("+EnumNInt::NInt", result_type=NInt_promoted.type.name)
        self.expect_expr("+EnumNLong::NLong", result_type=NLong_promoted.type.name)



import logging
import os
import os.path
import random

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.gdbclientutils import *
import lldbgdbserverutils
from lldbsuite.support import seven


class GDBProxyTestBase(TestBase):
    """
    Base class for gdbserver proxy tests.

    This class will setup and start a mock GDB server for the test to use.
    It pases through requests to a regular lldb-server/debugserver and
    forwards replies back to the LLDB under test.
    """

    """The gdbserver that we implement."""
    server = None
    """The inner lldb-server/debugserver process that we proxy requests into."""
    monitor_server = None
    monitor_sock = None

    server_socket_class = TCPServerSocket

    DEFAULT_TIMEOUT = 20 * (10 if ("ASAN_OPTIONS" in os.environ) else 1)

    _verbose_log_handler = None
    _log_formatter = logging.Formatter(fmt="%(asctime)-15s %(levelname)-8s %(message)s")

    def setUpBaseLogging(self):
        self.logger = logging.getLogger(__name__)

        self.logger.propagate = False
        self.logger.setLevel(logging.DEBUG)

        # log all warnings to stderr
        self._stderr_log_handler = logging.StreamHandler()
        self._stderr_log_handler.setLevel(
            logging.DEBUG if self.TraceOn() else logging.WARNING
        )
        self._stderr_log_handler.setFormatter(self._log_formatter)
        self.logger.addHandler(self._stderr_log_handler)

    def setUp(self):
        TestBase.setUp(self)

        self.setUpBaseLogging()

        if self.isVerboseLoggingRequested():
            # If requested, full logs go to a log file
            log_file_name = self.getLogBasenameForCurrentTest() + "-proxy.log"
            self._verbose_log_handler = logging.FileHandler(log_file_name)
            self._verbose_log_handler.setFormatter(self._log_formatter)
            self._verbose_log_handler.setLevel(logging.DEBUG)
            self.logger.addHandler(self._verbose_log_handler)

        if lldbplatformutil.getPlatform() == "macosx":
            self.debug_monitor_exe = lldbgdbserverutils.get_debugserver_exe()
            self.debug_monitor_extra_args = []
        else:
            self.debug_monitor_exe = lldbgdbserverutils.get_lldb_server_exe()
            self.debug_monitor_extra_args = ["gdbserver"]
        self.assertIsNotNone(self.debug_monitor_exe)

        self.server = MockGDBServer(self.server_socket_class())
        self.server.responder = self

    def tearDown(self):
        # TestBase.tearDown will kill the process, but we need to kill it early
        # so its client connection closes and we can stop the server before
        # finally calling the base tearDown.
        if self.process() is not None:
            self.process().Kill()
        self.server.stop()

        self.logger.removeHandler(self._verbose_log_handler)
        self._verbose_log_handler = None
        self.logger.removeHandler(self._stderr_log_handler)
        self._stderr_log_handler = None

        TestBase.tearDown(self)

    def isVerboseLoggingRequested(self):
        # We will report our detailed logs if the user requested that the "gdb-remote" channel is
        # logged.
        return any(("gdb-remote" in channel) for channel in lldbtest_config.channels)

    def connect(self, target):
        """
        Create a process by connecting to the mock GDB server.
        """
        self.prep_debug_monitor_and_inferior()
        self.server.start()

        listener = self.dbg.GetListener()
        error = lldb.SBError()
        process = target.ConnectRemote(
            listener, self.server.get_connect_url(), "gdb-remote", error
        )
        self.assertTrue(error.Success(), error.description)
        self.assertTrue(process, PROCESS_IS_VALID)
        return process

    def prep_debug_monitor_and_inferior(self):
        inferior_exe_path = self.getBuildArtifact("a.out")
        self.connect_to_debug_monitor([inferior_exe_path])
        self.assertIsNotNone(self.monitor_server)
        self.initial_handshake()

    def initial_handshake(self):
        self.monitor_server.send_packet(seven.bitcast_to_bytes("+"))
        reply = seven.bitcast_to_string(self.monitor_server.get_normal_packet())
        self.assertEqual(reply, "+")
        self.monitor_server.send_packet(seven.bitcast_to_bytes("QStartNoAckMode"))
        reply = seven.bitcast_to_string(self.monitor_server.get_normal_packet())
        self.assertEqual(reply, "+")
        reply = seven.bitcast_to_string(self.monitor_server.get_normal_packet())
        self.assertEqual(reply, "OK")
        self.monitor_server.set_validate_checksums(False)
        self.monitor_server.send_packet(seven.bitcast_to_bytes("+"))
        reply = seven.bitcast_to_string(self.monitor_server.get_normal_packet())
        self.assertEqual(reply, "+")

    def get_debug_monitor_command_line_args(self, connect_address, launch_args):
        return (
            self.debug_monitor_extra_args
            + ["--reverse-connect", connect_address]
            + launch_args
        )

    def launch_debug_monitor(self, launch_args):
        family, type, proto, _, addr = socket.getaddrinfo(
            "localhost", 0, proto=socket.IPPROTO_TCP
        )[0]
        sock = socket.socket(family, type, proto)
        sock.settimeout(self.DEFAULT_TIMEOUT)
        sock.bind(addr)
        sock.listen(1)
        addr = sock.getsockname()
        connect_address = "[{}]:{}".format(*addr)

        commandline_args = self.get_debug_monitor_command_line_args(
            connect_address, launch_args
        )

        # Start the server.
        self.logger.info(f"Spawning monitor {commandline_args}")
        monitor_process = self.spawnSubprocess(
            self.debug_monitor_exe, commandline_args, install_remote=False
        )
        self.assertIsNotNone(monitor_process)

        self.monitor_sock = sock.accept()[0]
        self.monitor_sock.settimeout(self.DEFAULT_TIMEOUT)
        return monitor_process

    def connect_to_debug_monitor(self, launch_args):
        monitor_process = self.launch_debug_monitor(launch_args)
        # Turn off checksum validation because debugserver does not produce
        # correct checksums.
        self.monitor_server = lldbgdbserverutils.Server(
            self.monitor_sock, monitor_process
        )

    def respond(self, packet):
        """Subclasses can override this to change how packets are handled."""
        return self.pass_through(packet)

    def pass_through(self, packet):
        self.logger.info(f"Sending packet {packet}")
        self.monitor_server.send_packet(seven.bitcast_to_bytes(packet))
        reply = seven.bitcast_to_string(self.monitor_server.get_normal_packet())
        self.logger.info(f"Received reply {reply}")
        return reply



"""A tool for extracting a list of private lldb symbols to export for MSVC.

When exporting symbols from a dll or exe we either need to mark the symbols in
the source code as __declspec(dllexport) or supply a list of symbols to the
linker. Private symbols in LLDB don't explicitly specific dllexport, so we
automate that by examining the symbol table.
"""

import argparse
import os
import re
import subprocess
import sys


def extract_symbols(nm_path: str, lib: str):
    """Extract all of the private lldb symbols from the given path to llvm-nm and
    library to extract from."""

    # Matches mangled symbols containing 'lldb_private'.
    lldb_sym_re = r"[0-9a-zA-Z]* [BT] (?P<symbol>[?]+[^?].*lldb_private.*)"

    # '-g' means we only get global symbols.
    # '-p' do not waste time sorting the symbols.
    process = subprocess.Popen(
        [nm_path, "-g", "-p", lib],
        bufsize=1,
        stdout=subprocess.PIPE,
        stdin=subprocess.PIPE,
        universal_newlines=True,
    )
    process.stdin.close()

    lldb_symbols = set()
    for line in process.stdout:
        match = re.match(lldb_sym_re, line)
        if match:
            symbol = match.group("symbol")
            assert (
                symbol.count(" ") == 0
            ), "Regex matched too much, probably got undecorated name as well"
            # Deleting destructors start with ?_G or ?_E and can be discarded
            # because link.exe gives you a warning telling you they can't be
            # exported if you don't.
            if symbol.startswith("??_G") or symbol.startswith("??_E"):
                continue
            lldb_symbols.add(symbol)

    return lldb_symbols


def main():
    parser = argparse.ArgumentParser(description="Generate LLDB dll exports")
    parser.add_argument(
        "-o", metavar="file", type=str, help="The name of the resultant export file."
    )
    parser.add_argument("--nm", help="Path to the llvm-nm executable.")
    parser.add_argument(
        "libs",
        metavar="lib",
        type=str,
        nargs="+",
        help="The libraries to extract symbols from.",
    )
    args = parser.parse_args()

    # Get the list of libraries to extract symbols from
    libs = list()
    for lib in args.libs:
        # When invoked by cmake the arguments are the cmake target names of the
        # libraries, so we need to add .lib/.a to the end and maybe lib to the
        # start to get the filename. Also allow objects.
        suffixes = [".lib", ".a", ".obj", ".o"]
        if not any([lib.endswith(s) for s in suffixes]):
            for suffix in suffixes:
                if os.path.exists(lib + suffix):
                    lib = lib + suffix
                    break
                if os.path.exists("lib" + lib + suffix):
                    lib = "lib" + lib + suffix
                    break
        if not any([lib.endswith(s) for s in suffixes]):
            print(
                "Unknown extension type for library argument: " + lib, file=sys.stderr
            )
            exit(1)
        libs.append(lib)

    # Extract symbols from the input libraries.
    symbols = set()
    for lib in libs:
        for sym in list(extract_symbols(args.nm, lib)):
            symbols.add(sym)

    # Write out the symbols to the output file.
    with open(args.o, "w", newline="") as f:
        for s in sorted(symbols):
            f.write(f"{s}\n")


if __name__ == "__main__":
    main()



"""
Test that variable expressions of type char are evaluated correctly.
"""

import AbstractBase

from lldbsuite.test.decorators import *


class CharTypeExprTestCase(AbstractBase.GenericTester):
    def test_char_type(self):
        """Test that char-type variable expressions are evaluated correctly."""
        self.build_and_run_expr("char.cpp", ["char"], qd=True)

    @skipUnlessDarwin
    def test_char_type_from_block(self):
        """Test that char-type variables are displayed correctly from a block."""
        self.build_and_run_expr("char.cpp", ["char"], bc=True, qd=True)

    def test_unsigned_char_type(self):
        """Test that 'unsigned_char'-type variable expressions are evaluated correctly."""
        self.build_and_run_expr("unsigned_char.cpp", ["unsigned", "char"], qd=True)

    @skipUnlessDarwin
    def test_unsigned_char_type_from_block(self):
        """Test that 'unsigned char'-type variables are displayed correctly from a block."""
        self.build_and_run_expr(
            "unsigned_char.cpp", ["unsigned", "char"], bc=True, qd=True
        )



"""
Test lldb-dap 'stopped' events.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbdap_testcase


@skipIfWindows  # This is flakey on Windows: llvm.org/pr24668, llvm.org/pr38373
@skipIfLinux
class TestDAP_stopped_events(lldbdap_testcase.DAPTestCaseBase):
    """
    Test validates different operations that produce 'stopped' events.
    """

    ANY_THREAD = {}

    def is_subdict(self, small: dict, big: dict) -> bool:
        """Returns true if 'small' is a subset of 'big', otherwise false."""
        return big | small == big

    def verify_threads(self, expected_threads):
        threads_resp = self.dap_server.request_threads()
        self.assertTrue(threads_resp["success"])
        threads = threads_resp["body"]["threads"]
        self.assertGreaterEqual(
            len(threads),
            len(expected_threads),
            f"thread: {threads!r}\n expected threads: {expected_threads}",
        )

        for idx, expected_thread in enumerate(expected_threads):
            thread = threads[idx]
            self.assertTrue(
                self.is_subdict(expected_thread, thread),
                f"Invalid thread state in {threads_resp}",
            )

    @expectedFailureAll(
        oslist=["freebsd"],
        bugnumber="llvm.org/pr18190 thread states not properly maintained",
    )
    @expectedFailureNetBSD
    def test_multiple_threads_same_breakpoint(self):
        """
        Test that multiple threads being stopped on the same breakpoint produces multiple 'stopped' event.
        """
        program = self.getBuildArtifact("a.out")
        launch_seq = self.build_and_launch(program)
        self.dap_server.wait_for_event(["initialized"])
        [bp] = self.set_function_breakpoints(["my_add"])
        self.verify_configuration_done()
        response = self.dap_server.receive_response(launch_seq)
        self.assertTrue(response["success"])

        events = self.dap_server.wait_for_stopped()
        if len(events) == 1:
            # we may not catch both events at the same time if we run with sanitizers
            # enabled or on slow or heavily used computer.
            events.extend(self.dap_server.wait_for_stopped())

        self.assertEqual(
            len(events),
            2,
            f"Expected two 'stopped' events, seen events: {events!r}",
        )
        for event in events:
            body = event["body"]
            self.assertEqual(body["reason"], "breakpoint")
            self.assertEqual(body["text"], "breakpoint 1.1")
            self.assertEqual(body["description"], "breakpoint 1.1")
            self.assertEqual(body["hitBreakpointIds"], [int(bp)])
            self.assertIsNotNone(body["threadId"])

        # We should have three threads, something along the lines of:
        #
        # Process 1234 stopped
        #   thread #1: tid = 0x01, 0x0a libsystem_pthread.dylib`pthread_mutex_lock + 12, queue = 'com.apple.main-thread'
        # * thread #2: tid = 0x02, 0x0b a.out`add(a=1, b=2) at main.cpp:10:32, stop reason = breakpoint 1.1
        #   thread #3: tid = 0x03, 0x0c a.out`add(a=4, b=5) at main.cpp:10:32, stop reason = breakpoint 1.1
        self.verify_threads(
            [
                self.ANY_THREAD,
                {
                    "reason": "breakpoint",
                    "text": "breakpoint 1.1",
                    "description": "breakpoint 1.1",
                },
                {
                    "reason": "breakpoint",
                    "text": "breakpoint 1.1",
                    "description": "breakpoint 1.1",
                },
            ]
        )

        self.assertEqual(
            self.dap_server.threads[1]["id"],
            self.dap_server.focused_tid,
            "Expected thread#2 to be focused",
        )

        self.continue_to_exit()

    @expectedFailureAll(
        oslist=["freebsd"],
        bugnumber="llvm.org/pr18190 thread states not properly maintained",
    )
    @expectedFailureNetBSD
    def test_multiple_breakpoints_same_location(self):
        """
        Test stopping at a location that reports multiple overlapping breakpoints.
        """
        program = self.getBuildArtifact("a.out")
        launch_seq = self.build_and_launch(program)
        self.dap_server.wait_for_event(["initialized"])
        line_1 = line_number("main.cpp", "breakpoint")
        [bp1] = self.set_source_breakpoints("main.cpp", [line_1])
        [bp2] = self.set_function_breakpoints(["my_add"])
        self.verify_configuration_done()
        response = self.dap_server.receive_response(launch_seq)
        self.assertTrue(response["success"])

        events = self.dap_server.wait_for_stopped()
        if len(events) == 1:
            # we may not catch both events at the same time if we run with sanitizers
            # enabled or on slow or heavily used computer.
            events.extend(self.dap_server.wait_for_stopped())

        self.assertEqual(len(events), 2, "Expected two stopped events")
        for event in events:
            body = event["body"]
            self.assertEqual(body["reason"], "breakpoint")
            self.assertEqual(body["text"], "breakpoint 1.1 2.1")
            self.assertEqual(body["description"], "breakpoint 1.1 2.1")
            self.assertEqual(body["hitBreakpointIds"], [int(bp1), int(bp2)])
            self.assertIsNotNone(body["threadId"])

        # Should return something like:
        # Process 1234 stopped
        #   thread #1: tid = 0x01, 0x0a libsystem_pthread.dylib`pthread_mutex_lock + 12, queue = 'com.apple.main-thread'
        # * thread #2: tid = 0x02, 0x0b a.out`add(a=1, b=2) at main.cpp:10:32, stop reason = breakpoint 1.1 2.1
        #   thread #3: tid = 0x03, 0x0c a.out`add(a=4, b=5) at main.cpp:10:32, stop reason = breakpoint 1.1 2.1
        self.verify_threads(
            [
                self.ANY_THREAD,
                {
                    "reason": "breakpoint",
                    "description": "breakpoint 1.1 2.1",
                    "text": "breakpoint 1.1 2.1",
                },
                {
                    "reason": "breakpoint",
                    "description": "breakpoint 1.1 2.1",
                    "text": "breakpoint 1.1 2.1",
                },
            ]
        )

        self.assertEqual(
            self.dap_server.threads[1]["id"],
            self.dap_server.focused_tid,
            "Expected thread#2 to be focused",
        )

        self.continue_to_exit()



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase

# This test ensures that LLDB correctly handles packets sent by MSPDebug.
# https://github.com/dlbeer/mspdebug


class MyResponder(MockGDBServerResponder):
    def qSupported(self, client_supported):
        return "PacketSize=4000"

    def setBreakpoint(self, packet):
        return "OK"

    def stopPackets():
        # Registers 3 to 15 are empty
        regs3to15 = "".join("%02x:%s;" % (i, "00000000") for i in range(3, 16))
        yield "T0500:00050000;01:00000000;02:00000000;" + regs3to15
        yield "T0500:10050000;01:baff0000;02:05000000;" + regs3to15
        yield "T0500:16050000;01:baff0000;02:05000000;" + regs3to15

    stopPacket = stopPackets()

    def haltReason(self):
        return next(self.stopPacket)

    def cont(self):
        return self.haltReason()

    # Memory dump
    def readMemory(self, addr, length):
        # Program memory
        if addr == 0x0400:
            return (
                ("ff" * 256)
                + "3140c0ff0c43b0121c05b01281010000b240d2043c051c423c0530413180020081430000b01210053150020030411c4330413c402a0030410c433041"
                + ("ff" * 196)
            )
        # Stack contents
        if addr == 0xFE00:
            return ("ff" * 442) + "280500000a05" + ("ff" * 62) + "0005"


class TestMSP430MSPDebug(GDBRemoteTestBase):
    @skipIfLLVMTargetMissing("MSP430")
    def test(self):
        """
        Test LLDB's MSP430 functionality.
        """
        target = self.createTarget("msp430.yaml")
        self.server.responder = MyResponder()

        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )
        num_threads = len(process.threads)
        self.assertEqual(num_threads, 1, "Only one thread")
        thread = process.GetThreadAtIndex(0)

        # Test if a breakpoint can be set
        bp = target.BreakpointCreateByName("func")
        self.assertTrue(bp.IsValid())
        bp.SetEnabled(True)
        self.assertTrue(bp.IsEnabled())

        # Test if the breakpoint address is resolved correctly
        self.assertEqual(bp.GetNumLocations(), 1, "Only one location")
        bp_loc = bp.GetLocationAtIndex(0)
        self.assertEqual(
            bp_loc.GetAddress().GetLoadAddress(target), 0x510, "Address of main"
        )

        # Test if the process stops at the breakpoint
        process.Continue()
        self.assertStopReason(
            thread.GetStopReason(), lldb.eStopReasonBreakpoint, "Hit a breakpoint"
        )

        # Check if disassembler works and the current function is "func"
        func = thread.GetFrameAtIndex(0).GetFunction()
        insts = func.GetInstructions(target)
        inst = insts.GetInstructionAtIndex(0)
        self.assertEqual(inst.GetMnemonic(target), "mov")
        self.assertEqual(inst.GetOperands(target), "#1234, &1340")

        # Test if thread can step a single instruction
        thread.StepInstruction(False)
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetPCAddress().GetLoadAddress(target),
            0x516,
            "Address of the next instruction",
        )

        # Test if registers are being set correctly
        registerSet = thread.GetFrameAtIndex(0).GetRegisters().GetValueAtIndex(0)
        reg_val_dict = {
            "pc": 0x0516,
            "sp": 0xFFBA,
            "r2": 0x0005,
            "r3": 0x0000,
            "fp": 0x0000,
            "r5": 0x0000,
            "r6": 0x0000,
            "r7": 0x0000,
            "r8": 0x0000,
            "r9": 0x0000,
            "r10": 0x0000,
            "r11": 0x0000,
            "r12": 0x0000,
            "r13": 0x0000,
            "r14": 0x0000,
            "r15": 0x0000,
        }
        for reg in registerSet:
            self.assertEqual(reg.GetValueAsUnsigned(), reg_val_dict[reg.GetName()])

        # Check if backtracing works:
        self.assertGreaterEqual(len(thread.frames), 3)
        crt0_addr = thread.GetFrameAtIndex(2).GetPCAddress().GetLoadAddress(target)
        self.assertEqual(crt0_addr, 0x50A)



import os, json, struct, signal

from typing import Any, Dict

import lldb
from lldb.plugins.scripted_process import ScriptedProcess
from lldb.plugins.scripted_process import ScriptedThread


class StackCoreScriptedProcess(ScriptedProcess):
    def get_module_with_name(self, target, name):
        for module in target.modules:
            if name in module.GetFileSpec().GetFilename():
                return module
        return None

    def __init__(self, exe_ctx: lldb.SBExecutionContext, args: lldb.SBStructuredData):
        super().__init__(exe_ctx, args)

        self.corefile_target = None
        self.corefile_process = None

        self.backing_target_idx = args.GetValueForKey("backing_target_idx")
        if self.backing_target_idx and self.backing_target_idx.IsValid():
            if self.backing_target_idx.GetType() == lldb.eStructuredDataTypeInteger:
                idx = self.backing_target_idx.GetIntegerValue(42)
            if self.backing_target_idx.GetType() == lldb.eStructuredDataTypeString:
                idx = int(self.backing_target_idx.GetStringValue(100))
            self.corefile_target = self.target.GetDebugger().GetTargetAtIndex(idx)
            self.corefile_process = self.corefile_target.GetProcess()
            for corefile_thread in self.corefile_process:
                structured_data = lldb.SBStructuredData()
                structured_data.SetFromJSON(
                    json.dumps(
                        {
                            "backing_target_idx": idx,
                            "thread_idx": corefile_thread.GetIndexID(),
                        }
                    )
                )

                self.threads[corefile_thread.GetThreadID()] = StackCoreScriptedThread(
                    self, structured_data
                )

        if len(self.threads) == 2:
            self.threads[len(self.threads) - 1].is_stopped = True

        custom_modules = args.GetValueForKey("custom_modules")
        if custom_modules.GetType() == lldb.eStructuredDataTypeArray:
            for id in range(custom_modules.GetSize()):

                custom_module = custom_modules.GetItemAtIndex(id)
                if (
                    not custom_module
                    or not custom_module.IsValid()
                    or not custom_module.GetType() == lldb.eStructuredDataTypeDictionary
                ):
                    continue

                # Get custom module path from args
                module_path_arg = custom_module.GetValueForKey("path")
                module_path = None
                if (
                    not module_path_arg
                    or not module_path_arg.IsValid()
                    or not module_path_arg.GetType() == lldb.eStructuredDataTypeString
                ):
                    continue

                module_path = module_path_arg.GetStringValue(100)

                # Get custom module load address from args
                module_load_addr = None
                module_load_addr_arg = custom_module.GetValueForKey("load_addr")
                if (
                    module_load_addr_arg
                    and module_load_addr_arg.IsValid()
                    and module_load_addr_arg.GetType()
                    == lldb.eStructuredDataTypeInteger
                ):
                    module_load_addr = module_load_addr_arg.GetIntegerValue()

                # If module load address is not specified/valid, try to find it from corefile module
                if module_load_addr is None:
                    module_name = os.path.basename(module_path)
                    corefile_module = self.get_module_with_name(
                        self.corefile_target, module_name
                    )

                    if not corefile_module or not corefile_module.IsValid():
                        continue

                    module_load_addr = (
                        corefile_module.GetObjectFileHeaderAddress().GetLoadAddress(
                            self.corefile_target
                        )
                    )

                self.loaded_images.append(
                    {
                        "path": module_path,
                        "load_addr": module_load_addr,
                    }
                )

    def get_memory_region_containing_address(
        self, addr: int
    ) -> lldb.SBMemoryRegionInfo:
        mem_region = lldb.SBMemoryRegionInfo()
        error = self.corefile_process.GetMemoryRegionInfo(addr, mem_region)
        if error.Fail():
            return None
        return mem_region

    def read_memory_at_address(
        self, addr: int, size: int, error: lldb.SBError
    ) -> lldb.SBData:
        data = lldb.SBData()
        bytes_read = self.corefile_process.ReadMemory(addr, size, error)

        if error.Fail():
            return data

        data.SetDataWithOwnership(
            error,
            bytes_read,
            self.corefile_target.GetByteOrder(),
            self.corefile_target.GetAddressByteSize(),
        )

        return data

    def get_loaded_images(self):
        return self.loaded_images

    def get_process_id(self) -> int:
        return 42

    def should_stop(self) -> bool:
        return True

    def is_alive(self) -> bool:
        return True

    def get_scripted_thread_plugin(self):
        return (
            StackCoreScriptedThread.__module__ + "." + StackCoreScriptedThread.__name__
        )


class StackCoreScriptedThread(ScriptedThread):
    def __init__(self, process, args):
        super().__init__(process, args)
        backing_target_idx = args.GetValueForKey("backing_target_idx")
        thread_idx = args.GetValueForKey("thread_idx")
        self.is_stopped = False

        def extract_value_from_structured_data(data, default_val):
            if data and data.IsValid():
                if data.GetType() == lldb.eStructuredDataTypeInteger:
                    return data.GetIntegerValue(default_val)
                if data.GetType() == lldb.eStructuredDataTypeString:
                    return int(data.GetStringValue(100))
            return None

        # TODO: Change to Walrus operator (:=) with oneline if assignment
        # Requires python 3.8
        val = extract_value_from_structured_data(thread_idx, 0)
        if val is not None:
            self.idx = val

        self.corefile_target = None
        self.corefile_process = None
        self.corefile_thread = None

        # TODO: Change to Walrus operator (:=) with oneline if assignment
        # Requires python 3.8
        val = extract_value_from_structured_data(backing_target_idx, 42)
        if val is not None:
            self.corefile_target = self.target.GetDebugger().GetTargetAtIndex(val)
            self.corefile_process = self.corefile_target.GetProcess()
            self.corefile_thread = self.corefile_process.GetThreadByIndexID(self.idx)

        if self.corefile_thread:
            self.id = self.corefile_thread.GetThreadID()

    def get_thread_id(self) -> int:
        return self.id

    def get_name(self) -> str:
        return StackCoreScriptedThread.__name__ + ".thread-" + str(self.id)

    def get_stop_reason(self) -> Dict[str, Any]:
        stop_reason = {"type": lldb.eStopReasonInvalid, "data": {}}

        if (
            self.corefile_thread
            and self.corefile_thread.IsValid()
            and self.get_thread_id() == self.corefile_thread.GetThreadID()
        ):
            stop_reason["type"] = lldb.eStopReasonNone

            if self.is_stopped:
                if "arm64" in self.scripted_process.arch:
                    stop_reason["type"] = lldb.eStopReasonException
                    stop_reason["data"]["desc"] = self.corefile_thread.stop_description
                elif self.scripted_process.arch == "x86_64":
                    stop_reason["type"] = lldb.eStopReasonSignal
                    stop_reason["data"]["signal"] = signal.SIGTRAP
                else:
                    stop_reason["type"] = self.corefile_thread.GetStopReason()

        return stop_reason

    def get_register_context(self) -> str:
        if not self.corefile_thread or self.corefile_thread.GetNumFrames() == 0:
            return None
        frame = self.corefile_thread.GetFrameAtIndex(0)

        GPRs = None
        registerSet = frame.registers  # Returns an SBValueList.
        for regs in registerSet:
            if "general purpose" in regs.name.lower():
                GPRs = regs
                break

        if not GPRs:
            return None

        for reg in GPRs:
            self.register_ctx[reg.name] = int(reg.value, base=16)

        return struct.pack(
            "{}Q".format(len(self.register_ctx)), *self.register_ctx.values()
        )


def __lldb_init_module(debugger, dict):
    if not "SKIP_SCRIPTED_PROCESS_LAUNCH" in os.environ:
        debugger.HandleCommand(
            "process launch -C %s.%s" % (__name__, StackCoreScriptedProcess.__name__)
        )
    else:
        print(
            "Name of the class that will manage the scripted process: '%s.%s'"
            % (__name__, StackCoreScriptedProcess.__name__)
        )



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class NSSetSyntheticTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.m", "// Set break point at this line.")

    @skipUnlessDarwin
    def test_rdar12529957_with_run_command(self):
        """Test that NSSet reports its synthetic children properly."""
        self.build()
        self.run_tests()

    @skipUnlessDarwin
    def test_rdar12529957_with_run_command_no_const(self):
        """Test that NSSet reports its synthetic children properly."""
        disable_constant_classes = {
            "CFLAGS_EXTRAS": "-fno-constant-nsnumber-literals "
            + "-fno-constant-nsarray-literals "
            + "-fno-constant-nsdictionary-literals",
        }
        # FIXME: Remove compiler when flags are available upstream.
        self.build(dictionary=disable_constant_classes, compiler="xcrun clang")
        self.run_tests()

    def run_tests(self):
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Now check that we are displaying Cocoa classes correctly
        self.expect("frame variable set", substrs=["4 elements"])
        self.expect("frame variable mutable", substrs=["9 elements"])
        self.expect(
            "frame variable set --ptr-depth 1 -d run -T",
            substrs=[
                "4 elements",
                "[0]",
                "hello",
                "[1]",
                "(int)2",
                "[2]",
                "(int)1",
                "[3]",
                "world",
            ],
        )
        self.expect(
            "frame variable mutable --ptr-depth 1 -d run -T",
            substrs=[
                "9 elements",
                "(int)5",
                '@"3 elements"',
                '@"www.apple.com"',
                "(int)3",
                '@"world"',
                "(int)4",
            ],
        )

        self.runCmd("next")
        self.expect("frame variable mutable", substrs=["0 elements"])

        self.runCmd("next")
        self.expect("frame variable mutable", substrs=["4 elements"])
        self.expect(
            "frame variable mutable --ptr-depth 1 -d run -T",
            substrs=[
                "4 elements",
                "[0]",
                "(int)1",
                "[1]",
                "(int)2",
                "[2]",
                "hello",
                "[3]",
                "world",
            ],
        )

        self.runCmd("next")
        self.expect("frame variable mutable", substrs=["4 elements"])
        self.expect(
            "frame variable mutable --ptr-depth 1 -d run -T",
            substrs=[
                "4 elements",
                "[0]",
                "(int)1",
                "[1]",
                "(int)2",
                "[2]",
                "hello",
                "[3]",
                "world",
            ],
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPtrAuthExpressions(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessArm64eSupported
    def test_static_function_pointer(self):
        """On arm64e, function pointers are automatically signed (PAC).
        Test that we can call a function through a static function pointer
        from the expression evaluator, which requires "fixing up" the pointer
        signing via the InjectPointerSigningFixups pass."""
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c", False)
        )

        self.expect_expr(
            "static int (*fp)(int, int) = &add; fp(5, 6);",
            result_type="int",
            result_value="11",
        )

        self.expect_expr(
            "static int (*fp)(int, int) = &mul; fp(4, 5);",
            result_type="int",
            result_value="20",
        )

    @skipUnlessArm64eSupported
    def test_indirect_call_through_caller(self):
        """Test that a function pointer passed to a debuggee function is
        correctly signed. The caller() function in the debuggee forces a
        genuine indirect call, preventing the compiler from folding the
        function pointer call into a direct call."""
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c", False)
        )

        self.expect_expr(
            "caller(add, 2, 3);",
            result_type="int",
            result_value="5",
        )

        self.expect_expr(
            "caller(mul, 3, 7);",
            result_type="int",
            result_value="21",
        )

    @skipUnlessArm64eSupported
    def test_debuggee_signed_pointer(self):
        """Test that a signed function pointer stored in the debuggee's memory
        can be read and called from a user expression. The global_fp variable
        is signed with the IB key (__ptrauth(1, 0, 0)), which is
        process-specific; this verifies that auth succeeds because expressions
        execute in the debuggee's process, not the debugger's."""
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c", False)
        )

        self.expect_expr(
            "global_fp(10, 20);",
            result_type="int",
            result_value="30",
        )

    @skipUnlessArm64eSupported
    def test_indirect_goto(self):
        """Test that computed gotos (GCC labels-as-values) work in the
        expression evaluator on arm64e, where -fptrauth-indirect-gotos signs
        label addresses and the indirect branch authenticates them."""
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c", False)
        )

        # Call a debuggee function that uses a computed-goto dispatch table.
        self.expect_expr(
            "indirect_goto_dispatch(0)",
            result_type="int",
            result_value="10",
        )
        self.expect_expr(
            "indirect_goto_dispatch(1)",
            result_type="int",
            result_value="20",
        )
        self.expect_expr(
            "indirect_goto_dispatch(2)",
            result_type="int",
            result_value="30",
        )

        # Evaluate a computed goto directly in a user expression.
        # Use individual variables (not an array) so that the label addresses
        # are signed inline with pacia/braa instructions, avoiding @AUTH
        # relocations in global constant tables that RuntimeDyld cannot handle.
        self.expect_expr(
            "({ int result; void *t0 = &&L0, *t1 = &&L1, *t2 = &&L2; "
            "goto *t1; "
            "L0: result = 100; goto Lend; "
            "L1: result = 200; goto Lend; "
            "L2: result = 300; goto Lend; "
            "Lend: result; })",
            result_type="int",
            result_value="200",
        )



"""
Test that LLDB correctly handles fields
marked with [[no_unique_address]].
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class NoUniqueAddressTestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "return 0", lldb.SBFileSpec("main.cpp", False)
        )

        # Qualified/unqualified lookup to templates in namespace
        self.expect_expr(
            "b1",
            result_type="basic::Foo",
            result_children=[ValueCheck(name="a", type="Empty")],
        )

        self.expect_expr(
            "b2",
            result_type="bases::Foo",
            result_children=[
                ValueCheck(
                    type="bases::B", children=[ValueCheck(name="x", type="Empty")]
                ),
                ValueCheck(
                    type="bases::A",
                    children=[
                        ValueCheck(name="c", type="long", value="1"),
                        ValueCheck(name="d", type="long", value="2"),
                    ],
                ),
                ValueCheck(
                    type="bases::C", children=[ValueCheck(name="x", type="Empty")]
                ),
            ],
        )
        self.expect_expr(
            "b3",
            result_type="bases::Bar",
            result_children=[
                ValueCheck(
                    type="bases::B", children=[ValueCheck(name="x", type="Empty")]
                ),
                ValueCheck(
                    type="bases::C", children=[ValueCheck(name="x", type="Empty")]
                ),
                ValueCheck(
                    type="bases::A",
                    children=[
                        ValueCheck(name="c", type="long", value="5"),
                        ValueCheck(name="d", type="long", value="6"),
                    ],
                ),
            ],
        )

        self.expect("frame var b1")
        self.expect("frame var b2")
        self.expect("frame var b3")



"""Test the RunCommandInterpreter API."""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class CommandRunInterpreterLegacyAPICase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)

        self.stdin_path = self.getBuildArtifact("stdin.txt")

        with open(self.stdin_path, "w") as input_handle:
            input_handle.write("nonexistingcommand\nquit")

        # Python will close the file descriptor if all references
        # to the filehandle object lapse, so we need to keep one
        # around.
        self.filehandle = open(self.stdin_path, "r")
        self.dbg.SetInputFileHandle(self.filehandle, False)

        # No need to track the output
        self.devnull = open(os.devnull, "w")
        self.dbg.SetOutputFileHandle(self.devnull, False)
        self.dbg.SetErrorFileHandle(self.devnull, False)

    def test_run_session_with_error_and_quit_legacy(self):
        """Run non-existing and quit command returns appropriate values"""

        n_errors, quit_requested, has_crashed = self.dbg.RunCommandInterpreter(
            True, False, lldb.SBCommandInterpreterRunOptions(), 0, False, False
        )

        self.assertGreater(n_errors, 0)
        self.assertTrue(quit_requested)
        self.assertFalse(has_crashed)


class CommandRunInterpreterAPICase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)

        self.stdin_path = self.getBuildArtifact("stdin.txt")
        self.stdout_path = self.getBuildArtifact("stdout.txt")

    def run_commands_string(
        self, command_string, options=lldb.SBCommandInterpreterRunOptions()
    ):
        """Run the commands in command_string through RunCommandInterpreter.
        Returns (n_errors, quit_requested, has_crashed, result_string)."""

        with open(self.stdin_path, "w") as input_handle:
            input_handle.write(command_string)

        n_errors = 0
        quit_requested = False
        has_crashed = False

        with open(self.stdin_path, "r") as in_fileH, open(
            self.stdout_path, "w"
        ) as out_fileH:
            self.dbg.SetInputFile(in_fileH)

            self.dbg.SetOutputFile(out_fileH)
            self.dbg.SetErrorFile(out_fileH)

            n_errors, quit_requested, has_crashed = self.dbg.RunCommandInterpreter(
                True, False, options, 0, False, False
            )

        result_string = None
        with open(self.stdout_path, "r") as out_fileH:
            result_string = out_fileH.read()

        return (n_errors, quit_requested, has_crashed, result_string)

    def test_run_session_with_error_and_quit(self):
        """Run non-existing and quit command returns appropriate values"""

        n_errors, quit_requested, has_crashed, _ = self.run_commands_string(
            "nonexistingcommand\nquit\n"
        )
        self.assertGreater(n_errors, 0)
        self.assertTrue(quit_requested)
        self.assertFalse(has_crashed)

    def test_allow_repeat(self):
        """Try auto-repeat of process launch - the command will fail and
        the auto-repeat will fail because of no auto-repeat."""
        options = lldb.SBCommandInterpreterRunOptions()
        options.SetEchoCommands(False)
        options.SetAllowRepeats(True)

        n_errors, quit_requested, has_crashed, result_str = self.run_commands_string(
            "process launch\n\n", options
        )
        self.assertEqual(n_errors, 2)
        self.assertFalse(quit_requested)
        self.assertFalse(has_crashed)

        self.assertIn("invalid target", result_str)
        self.assertIn("no auto repeat", result_str)


class SBCommandInterpreterRunOptionsCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_command_interpreter_run_options(self):
        """Test SBCommandInterpreterRunOptions default values, getters & setters"""

        opts = lldb.SBCommandInterpreterRunOptions()

        # Check getters with default values
        self.assertFalse(opts.GetStopOnContinue())
        self.assertFalse(opts.GetStopOnError())
        self.assertFalse(opts.GetStopOnCrash())
        self.assertTrue(opts.GetEchoCommands())
        self.assertTrue(opts.GetPrintResults())
        self.assertTrue(opts.GetPrintErrors())
        self.assertTrue(opts.GetAddToHistory())
        self.assertFalse(opts.GetAllowRepeats())

        # Invert values
        opts.SetStopOnContinue(not opts.GetStopOnContinue())
        opts.SetStopOnError(not opts.GetStopOnError())
        opts.SetStopOnCrash(not opts.GetStopOnCrash())
        opts.SetEchoCommands(not opts.GetEchoCommands())
        opts.SetPrintResults(not opts.GetPrintResults())
        opts.SetPrintErrors(not opts.GetPrintErrors())
        opts.SetAddToHistory(not opts.GetAddToHistory())
        opts.SetAllowRepeats(not opts.GetAllowRepeats())

        # Check the value changed
        self.assertTrue(opts.GetStopOnContinue())
        self.assertTrue(opts.GetStopOnError())
        self.assertTrue(opts.GetStopOnCrash())
        self.assertFalse(opts.GetEchoCommands())
        self.assertFalse(opts.GetPrintResults())
        self.assertFalse(opts.GetPrintErrors())
        self.assertFalse(opts.GetAddToHistory())
        self.assertTrue(opts.GetAllowRepeats())



"""
Test basic std::vector<bool> functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBoolVector(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(bugnumber="rdar://100741983")
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        vector_type = "std::vector<bool>"
        size_type = "size_type"

        self.runCmd("settings set target.import-std-module true")

        self.expect_expr(
            "a",
            result_type=vector_type,
            result_children=[
                ValueCheck(value="false"),
                ValueCheck(value="true"),
                ValueCheck(value="false"),
                ValueCheck(value="true"),
            ],
        )
        self.expect_expr("a.size()", result_type=size_type, result_value="4")
        # FIXME: Without the casting the result can't be materialized.
        self.expect_expr("(bool)a.front()", result_type="bool", result_value="false")
        self.expect_expr("(bool)a[1]", result_type="bool", result_value="true")
        self.expect_expr("(bool)a.back()", result_type="bool", result_value="true")

        self.expect_expr("(bool)*a.begin()", result_type="bool", result_value="false")
        self.expect_expr("(bool)*a.rbegin()", result_type="bool", result_value="true")



from lldbsuite.test.lldbtest import *
import os
import time
import json

ADDRESS_REGEX = "0x[0-9a-fA-F]*"


# Decorator that runs a test with both modes of USE_SB_API.
# It assumes that no tests can be executed in parallel.
def testSBAPIAndCommands(func):
    def wrapper(*args, **kwargs):
        TraceIntelPTTestCaseBase.USE_SB_API = True
        func(*args, **kwargs)
        TraceIntelPTTestCaseBase.USE_SB_API = False
        func(*args, **kwargs)

    return wrapper


def skipIfNoIntelPT(func):
    """Skip tests if the system does not support tracing."""

    supported = os.path.exists("/sys/bus/event_source/devices/intel_pt/type")
    return unittest.skipIf(not supported, "intel-pt tracing is unsupported")(func)


# Class that should be used by all python Intel PT tests.
#
# It has a handy check that skips the test if the intel-pt plugin is not enabled.
#
# It also contains many functions that can test both the SB API or the command line version
# of the most important tracing actions.
class TraceIntelPTTestCaseBase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # If True, the trace test methods will use the SB API, otherwise they'll use raw commands.
    USE_SB_API = False

    def setUp(self):
        TestBase.setUp(self)
        if "intel-pt" not in configuration.enabled_plugins:
            self.skipTest("The intel-pt test plugin is not enabled")

    def skipIfPerCpuTracingIsNotSupported(self):
        def is_supported():
            try:
                with open("/proc/sys/kernel/perf_event_paranoid", "r") as permissions:
                    value = int(permissions.readlines()[0])
                    if value <= 0:
                        return True
            except:
                return False

        if not is_supported():
            self.skipTest(
                "Per cpu tracing is not supported. You need "
                "/proc/sys/kernel/perf_event_paranoid to be 0 or -1. "
                "You can use `sudo sysctl -w kernel.perf_event_paranoid=-1` for that."
            )

    def getTraceOrCreate(self):
        if not self.target().GetTrace().IsValid():
            error = lldb.SBError()
            self.target().CreateTrace(error)
        return self.target().GetTrace()

    def assertSBError(self, sberror, error=False):
        if error:
            self.assertTrue(sberror.Fail())
        else:
            self.assertSuccess(sberror)

    def createConfiguration(
        self,
        iptTraceSize=None,
        processBufferSizeLimit=None,
        enableTsc=False,
        psbPeriod=None,
        perCpuTracing=False,
    ):
        obj = {}
        if processBufferSizeLimit is not None:
            obj["processBufferSizeLimit"] = processBufferSizeLimit
        if iptTraceSize is not None:
            obj["iptTraceSize"] = iptTraceSize
        if psbPeriod is not None:
            obj["psbPeriod"] = psbPeriod
        obj["enableTsc"] = enableTsc
        obj["perCpuTracing"] = perCpuTracing

        configuration = lldb.SBStructuredData()
        configuration.SetFromJSON(json.dumps(obj))
        return configuration

    def traceStartThread(
        self,
        thread=None,
        error=False,
        substrs=None,
        iptTraceSize=None,
        enableTsc=False,
        psbPeriod=None,
    ):
        if self.USE_SB_API:
            trace = self.getTraceOrCreate()
            thread = thread if thread is not None else self.thread()
            configuration = self.createConfiguration(
                iptTraceSize=iptTraceSize, enableTsc=enableTsc, psbPeriod=psbPeriod
            )
            self.assertSBError(trace.Start(thread, configuration), error)
        else:
            command = "thread trace start"
            if thread is not None:
                command += " " + str(thread.GetIndexID())
            if iptTraceSize is not None:
                command += " -s " + str(iptTraceSize)
            if enableTsc:
                command += " --tsc"
            if psbPeriod is not None:
                command += " --psb-period " + str(psbPeriod)
            self.expect(command, error=error, substrs=substrs)

    def traceStartProcess(
        self,
        processBufferSizeLimit=None,
        error=False,
        substrs=None,
        enableTsc=False,
        psbPeriod=None,
        perCpuTracing=False,
    ):
        if self.USE_SB_API:
            trace = self.getTraceOrCreate()
            configuration = self.createConfiguration(
                processBufferSizeLimit=processBufferSizeLimit,
                enableTsc=enableTsc,
                psbPeriod=psbPeriod,
                perCpuTracing=perCpuTracing,
            )
            self.assertSBError(trace.Start(configuration), error=error)
        else:
            command = "process trace start"
            if processBufferSizeLimit is not None:
                command += " -l " + str(processBufferSizeLimit)
            if enableTsc:
                command += " --tsc"
            if psbPeriod is not None:
                command += " --psb-period " + str(psbPeriod)
            if perCpuTracing:
                command += " --per-cpu-tracing"
            self.expect(command, error=error, substrs=substrs)

    def traceStopProcess(self):
        if self.USE_SB_API:
            self.assertSuccess(self.target().GetTrace().Stop())
        else:
            self.expect("process trace stop")

    def traceStopThread(self, thread=None, error=False, substrs=None):
        if self.USE_SB_API:
            thread = thread if thread is not None else self.thread()
            self.assertSBError(self.target().GetTrace().Stop(thread), error)

        else:
            command = "thread trace stop"
            if thread is not None:
                command += " " + str(thread.GetIndexID())
            self.expect(command, error=error, substrs=substrs)

    def traceLoad(self, traceDescriptionFilePath, error=False, substrs=None):
        if self.USE_SB_API:
            traceDescriptionFile = lldb.SBFileSpec(traceDescriptionFilePath, True)
            loadTraceError = lldb.SBError()
            self.dbg.LoadTraceFromFile(loadTraceError, traceDescriptionFile)
            self.assertSBError(loadTraceError, error)
        else:
            command = f"trace load -v {traceDescriptionFilePath}"
            self.expect(command, error=error, substrs=substrs)

    def traceSave(self, traceBundleDir, compact=False, error=False, substrs=None):
        if self.USE_SB_API:
            save_error = lldb.SBError()
            self.target().GetTrace().SaveToDisk(
                save_error, lldb.SBFileSpec(traceBundleDir), compact
            )
            self.assertSBError(save_error, error)
        else:
            command = f"trace save {traceBundleDir}"
            if compact:
                command += " -c"
            self.expect(command, error=error, substrs=substrs)



"""
Test display and Python APIs on file and class static variables.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StaticVariableTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_with_run_command(self):
        """Test that file and class static variables display correctly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Global variables are no longer displayed with the "frame variable"
        # command.
        self.expect(
            "target variable A::g_points",
            VARIABLES_DISPLAYED_CORRECTLY,
            patterns=[r"\(PointType\[[1-9]*\]\) A::g_points = {"],
        )
        self.expect(
            "target variable g_points",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["(PointType[2]) g_points"],
        )

        # On Mac OS X, gcc 4.2 emits the wrong debug info for A::g_points.
        # A::g_points is an array of two elements.
        if self.platformIsDarwin() or self.getPlatform() == "linux":
            self.expect(
                "target variable A::g_points[1].x",
                VARIABLES_DISPLAYED_CORRECTLY,
                startstr="(int) A::g_points[1].x = 11",
            )

    @expectedFailureAll(
        compiler=["gcc"], bugnumber="Compiler emits incomplete debug info"
    )
    @expectedFailureAll(
        compiler=["clang"], compiler_version=["<", "3.9"], bugnumber="llvm.org/pr20550"
    )
    def test_with_run_command_complete(self):
        """
        Test that file and class static variables display correctly with
        complete debug information.
        """
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        # Global variables are no longer displayed with the "frame variable"
        # command.
        self.expect(
            "target variable A::g_points",
            VARIABLES_DISPLAYED_CORRECTLY,
            patterns=[
                r"\(PointType\[[1-9]*\]\) A::g_points = {",
                "(x = 1, y = 2)",
                "(x = 11, y = 22)",
            ],
        )

        # Ensure that we take the context into account and only print
        # A::g_points.
        self.expect(
            "target variable A::g_points",
            VARIABLES_DISPLAYED_CORRECTLY,
            matching=False,
            patterns=["(x = 3, y = 4)", "(x = 33, y = 44)"],
        )

        # Finally, ensure that we print both points when not specifying a
        # context.
        self.expect(
            "target variable g_points",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=[
                "(PointType[2]) g_points",
                "(x = 1, y = 2)",
                "(x = 11, y = 22)",
                "(x = 3, y = 4)",
                "(x = 33, y = 44)",
            ],
        )

    def build_value_check(self, var_name, values):
        children_1 = [
            ValueCheck(name="x", value=values[0], type="int"),
            ValueCheck(name="y", value=values[1], type="int"),
        ]
        children_2 = [
            ValueCheck(name="x", value=values[2], type="int"),
            ValueCheck(name="y", value=values[3], type="int"),
        ]
        elem_0 = ValueCheck(
            name="[0]", value=None, type="PointType", children=children_1
        )
        elem_1 = ValueCheck(
            name="[1]", value=None, type="PointType", children=children_2
        )
        value_check = ValueCheck(
            name=var_name, value=None, type="PointType[2]", children=[elem_0, elem_1]
        )

        return value_check

    @expectedFailureAll(
        compiler=["gcc"], bugnumber="Compiler emits incomplete debug info"
    )
    @expectedFailureAll(
        compiler=["clang"], compiler_version=["<", "3.9"], bugnumber="llvm.org/pr20550"
    )
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24764")
    @add_test_categories(["pyapi"])
    def test_with_python_FindValue(self):
        """Test Python APIs on file and class static variables."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByLocation("main.cpp", self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # Get the SBValue of 'A::g_points' and 'g_points'.
        frame = thread.GetFrameAtIndex(0)

        # arguments =>     False
        # locals =>        False
        # statics =>       True
        # in_scope_only => False
        valList = frame.GetVariables(False, False, True, False)

        # Build ValueCheckers for the values we're going to find:
        value_check_A = self.build_value_check("A::g_points", ["1", "2", "11", "22"])
        value_check_none = self.build_value_check("g_points", ["3", "4", "33", "44"])
        value_check_AA = self.build_value_check("AA::g_points", ["5", "6", "55", "66"])

        for val in valList:
            self.DebugSBValue(val)
            name = val.GetName()
            self.assertIn(name, ["g_points", "A::g_points", "AA::g_points"])

            if name == "A::g_points":
                self.assertEqual(val.GetValueType(), lldb.eValueTypeVariableGlobal)
                value_check_A.check_value(self, val, "Got A::g_points right")
            if name == "g_points":
                self.assertEqual(val.GetValueType(), lldb.eValueTypeVariableStatic)
                value_check_none.check_value(self, val, "Got g_points right")
            if name == "AA::g_points":
                self.assertEqual(val.GetValueType(), lldb.eValueTypeVariableGlobal)
                value_check_AA.check_value(self, val, "Got AA::g_points right")

        # SBFrame.FindValue() should also work.
        val = frame.FindValue("A::g_points", lldb.eValueTypeVariableGlobal)
        self.DebugSBValue(val)
        value_check_A.check_value(self, val, "FindValue also works")

        # Also exercise the "parameter" and "local" scopes while we are at it.
        val = frame.FindValue("argc", lldb.eValueTypeVariableArgument)
        self.DebugSBValue(val)
        self.assertEqual(val.GetName(), "argc")

        val = frame.FindValue("argv", lldb.eValueTypeVariableArgument)
        self.DebugSBValue(val)
        self.assertEqual(val.GetName(), "argv")

        val = frame.FindValue("hello_world", lldb.eValueTypeVariableLocal)
        self.DebugSBValue(val)
        self.assertEqual(val.GetName(), "hello_world")

    # This test tests behavior that's been broken for a very long time..
    # The fix for it is in the accelerator table part of the DWARF reader,
    # and I fixed the version that the names accelerator uses, but I don't
    # know how to fix it on systems that don't use that. There isn't a
    # decorator for that - not sure how to construct that so I'm limiting the
    # test do Darwin for now.
    @expectedFailureAll(
        compiler=["gcc"], bugnumber="Compiler emits incomplete debug info"
    )
    @skipUnlessDarwin
    @add_test_categories(["pyapi"])
    def test_with_python_FindGlobalVariables(self):
        """Test Python APIs on file and class static variables."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByLocation("main.cpp", self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # Get the SBValue of 'A::g_points' and 'g_points'.
        frame = thread.GetFrameAtIndex(0)

        # Build ValueCheckers for the values we're going to find:
        value_check_A = self.build_value_check("A::g_points", ["1", "2", "11", "22"])
        value_check_none = self.build_value_check("g_points", ["3", "4", "33", "44"])
        value_check_AA = self.build_value_check("AA::g_points", ["5", "6", "55", "66"])

        # We should also be able to get class statics from FindGlobalVariables.
        # eMatchTypeStartsWith should only find A:: not AA::
        val_list = target.FindGlobalVariables("A::", 10, lldb.eMatchTypeStartsWith)
        self.assertEqual(val_list.GetSize(), 1, "Found only one match")
        val = val_list[0]
        value_check_A.check_value(self, val, "FindGlobalVariables starts with")

        # Regex should find both
        val_list = target.FindGlobalVariables("A::", 10, lldb.eMatchTypeRegex)
        self.assertEqual(val_list.GetSize(), 2, "Found A & AA")
        found_a = False
        found_aa = False
        for val in val_list:
            name = val.GetName()
            if name == "A::g_points":
                value_check_A.check_value(self, val, "AA found by regex")
                found_a = True
            elif name == "AA::g_points":
                value_check_AA.check_value(self, val, "A found by regex")
                found_aa = True

        self.assertTrue(found_a, "Regex search found A::g_points")
        self.assertTrue(found_aa, "Regex search found AA::g_points")

        # Regex lowercase should find both as well.
        val_list = target.FindGlobalVariables(
            "a::g_points", 10, lldb.eMatchTypeRegexInsensitive
        )
        self.assertEqual(val_list.GetSize(), 2, "Found A & AA")

        # Normal search for full name should find one, but it looks like we don't match
        # on identifier boundaries here yet:
        val_list = target.FindGlobalVariables("A::g_points", 10, lldb.eMatchTypeNormal)
        self.assertEqual(
            val_list.GetSize(), 2, "We aren't matching on name boundaries yet"
        )

        # Normal search for g_points should find 3 - FindGlobalVariables doesn't distinguish
        # between file statics and globals:
        val_list = target.FindGlobalVariables("g_points", 10, lldb.eMatchTypeNormal)
        self.assertEqual(val_list.GetSize(), 3, "Found all three g_points")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test_invalid_arg(self):
        self.expect(
            "target stop-hook disable -1",
            error=True,
            startstr='error: invalid stop hook id: "-1".',
        )
        self.expect(
            "target stop-hook disable abcdfx",
            error=True,
            startstr='error: invalid stop hook id: "abcdfx".',
        )



"""
Test that thread plan listing, and deleting works.
"""


import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestThreadPlanCommands(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfWindows
    def test_thread_plan_actions(self):
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.thread_plan_test()

    def check_list_output(
        self, command, active_plans=[], completed_plans=[], discarded_plans=[]
    ):
        # Check the "thread plan list" output against a list of active & completed and discarded plans.
        # If all three check arrays are empty, that means the command is expected to fail.

        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()

        num_active = len(active_plans)
        num_completed = len(completed_plans)
        num_discarded = len(discarded_plans)

        interp.HandleCommand(command, result)
        print("Command: %s" % (command))
        print(result.GetOutput())

        if num_active == 0 and num_completed == 0 and num_discarded == 0:
            self.assertFalse(
                result.Succeeded(),
                "command: '%s' succeeded when it should have failed: '%s'"
                % (command, result.GetError()),
            )
            return

        self.assertTrue(
            result.Succeeded(),
            "command: '%s' failed: '%s'" % (command, result.GetError()),
        )
        result_arr = result.GetOutput().splitlines()
        num_results = len(result_arr)

        # Now iterate through the results array and pick out the results.
        result_idx = 0
        self.assertIn("thread #", result_arr[result_idx], "Found thread header")
        result_idx += 1
        self.assertIn(
            "Active plan stack", result_arr[result_idx], "Found active header"
        )
        result_idx += 1
        self.assertIn(
            "Element 0: Base thread plan", result_arr[result_idx], "Found base plan"
        )
        result_idx += 1

        for text in active_plans:
            self.assertIn(
                text, result_arr[result_idx], "Didn't find active plan: %s" % (text)
            )
            result_idx += 1

        if len(completed_plans) > 0:
            # First consume any remaining active plans:
            while not "Completed plan stack:" in result_arr[result_idx]:
                result_idx += 1
                if result_idx == num_results:
                    self.fail(
                        "There should have been completed plans, but I never saw the completed stack header"
                    )
            # We are at the Completed header, skip it:
            result_idx += 1
            for text in completed_plans:
                self.assertIn(
                    text,
                    result_arr[result_idx],
                    "Didn't find completed plan: %s" % (text),
                )
                result_idx += 1

        if len(discarded_plans) > 0:
            # First consume any remaining completed plans:
            while not "Discarded plan stack:" in result_arr[result_idx]:
                result_idx += 1
                if result_idx == num_results:
                    self.fail(
                        "There should have been discarded plans, but I never saw the discarded stack header"
                    )

            # We are at the Discarded header, skip it:
            result_idx += 1
            for text in discarded_plans:
                self.assertIn(
                    text,
                    result_arr[result_idx],
                    "Didn't find discarded plan: %s" % (text),
                )
                result_idx += 1

    def thread_plan_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        # We need to have an internal plan so we can test listing one.
        # The most consistent way to do that is to use a scripted thread plan
        # that uses a sub-plan.  Source that in now.
        source_path = os.path.join(self.getSourceDir(), "wrap_step_over.py")
        self.runCmd("command script import '%s'" % (source_path))

        # Now set a breakpoint that we will hit by running our scripted step.
        call_me_bkpt = target.BreakpointCreateBySourceRegex(
            "Set another here", self.main_source_file
        )
        self.assertGreater(
            call_me_bkpt.GetNumLocations(), 0, "Set the breakpoint successfully"
        )
        thread.StepUsingScriptedThreadPlan("wrap_step_over.WrapStepOver")
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, call_me_bkpt)
        self.assertEqual(len(threads), 1, "Hit my breakpoint while stepping over")

        current_id = threads[0].GetIndexID()
        current_tid = threads[0].GetThreadID()
        # Run thread plan list without the -i flag:
        command = "thread plan list %d" % (current_id)
        self.check_list_output(command, ["wrap_step_over.WrapStepOver"], [])

        # Run thread plan list with the -i flag:
        command = "thread plan list -i %d" % (current_id)
        self.check_list_output(command, ["WrapStepOver", "Stepping over line main.c"])

        # Run thread plan list providing TID, output should be the same:
        command = "thread plan list -t %d" % (current_tid)
        self.check_list_output(command, ["wrap_step_over.WrapStepOver"])

        # Provide both index & tid, and make sure we only print once:
        command = "thread plan list -t %d %d" % (current_tid, current_id)
        self.check_list_output(command, ["wrap_step_over.WrapStepOver"])

        # Try a fake TID, and make sure that fails:
        fake_tid = 0
        for i in range(100, 10000, 100):
            fake_tid = current_tid + i
            thread = process.GetThreadByID(fake_tid)
            if not thread:
                break

        command = "thread plan list -t %d" % (fake_tid)
        self.check_list_output(command)

        # Now continue, and make sure we printed the completed plan:
        process.Continue()
        threads = lldbutil.get_stopped_threads(process, lldb.eStopReasonPlanComplete)
        self.assertEqual(len(threads), 1, "One thread completed a step")

        # Run thread plan list - there aren't any private plans at this point:
        command = "thread plan list %d" % (current_id)
        self.check_list_output(command, [], ["wrap_step_over.WrapStepOver"])

        # Set another breakpoint that we can run to, to try deleting thread plans.
        second_step_bkpt = target.BreakpointCreateBySourceRegex(
            "Run here to step over again", self.main_source_file
        )
        self.assertGreater(
            second_step_bkpt.GetNumLocations(), 0, "Set the breakpoint successfully"
        )
        final_bkpt = target.BreakpointCreateBySourceRegex(
            "Make sure we get here on last continue", self.main_source_file
        )
        self.assertGreater(
            final_bkpt.GetNumLocations(), 0, "Set the breakpoint successfully"
        )

        threads = lldbutil.continue_to_breakpoint(process, second_step_bkpt)
        self.assertEqual(len(threads), 1, "Hit the second step breakpoint")

        threads[0].StepOver()
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, call_me_bkpt)

        result = lldb.SBCommandReturnObject()
        interp = self.dbg.GetCommandInterpreter()
        interp.HandleCommand("thread plan discard 1", result)
        self.assertTrue(
            result.Succeeded(), "Deleted the step over plan: %s" % (result.GetOutput())
        )

        # Make sure the plan gets listed in the discarded plans:
        command = "thread plan list %d" % (current_id)
        self.check_list_output(command, [], [], ["Stepping over line main.c:"])

        process.Continue()
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, final_bkpt)
        self.assertEqual(len(threads), 1, "Ran to final breakpoint")
        threads = lldbutil.get_stopped_threads(process, lldb.eStopReasonPlanComplete)
        self.assertEqual(len(threads), 0, "Did NOT complete the step over plan")



import re

from lldbsuite.test.lldbtest import TestBase
from lldbsuite.test.decorators import *


def expectedFailureDwarf(bugnumber=None):
    return expectedFailureAll(bugnumber, debug_info="dwarf")


class TestDecoratorsNoDebugInfoClass(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(debug_info="dwarf")
    def test_decorator_xfail(self):
        """Test that specifying a debug info category works for a NO_DEBUG_INFO_TESTCASE"""

    @expectedFailureDwarf
    def test_decorator_xfail_bare_decorator(self):
        """Same as test_decorator_xfail, but with a custom decorator w/ a bare syntax"""

    @expectedFailureDwarf()
    def test_decorator_xfail_decorator_empty_args(self):
        """Same as test_decorator_xfail, but with a custom decorator w/ no args"""

    @add_test_categories(["dwarf"])
    def test_add_test_categories(self):
        # Note: the "dwarf" test category is ignored, because we don't generate any debug info test variants
        self.assertIsNone(self.getDebugInfo())

    @expectedFailureAll
    def test_xfail_empty(self):
        """Test that expectedFailureAll can be empty (but please just use expectedFailure)"""
        self.fail()

    @expectedFailureAll(compiler=re.compile(".*"))
    def test_xfail_regexp(self):
        """Test that xfail can take a regex as a matcher"""
        self.fail()

    @expectedFailureAll(compiler=no_match(re.compile(".*")))
    def test_xfail_no_match(self):
        """Test that xfail can take a no_match matcher"""
        pass

    @expectedFailureIf(condition=True)
    def test_xfail_condition_true(self):
        self.fail()

    @expectedFailureIf(condition=False)
    def test_xfail_condition_false(self):
        pass


class TestDecorators(TestBase):
    @expectedFailureAll(debug_info="dwarf")
    def test_decorator_xfail(self):
        """Test that expectedFailureAll fails for the debug_info variant"""
        if self.getDebugInfo() == "dwarf":
            self.fail()

    @skipIf(debug_info="dwarf")
    def test_decorator_skip(self):
        """Test that skipIf skips the debug_info variant"""
        self.assertNotEqual(self.getDebugInfo(), "dwarf")

    @expectedFailureDwarf
    def test_decorator_xfail2(self):
        """Same as test_decorator_xfail, but with a custom decorator w/ a bare syntax"""
        if self.getDebugInfo() == "dwarf":
            self.fail()

    @expectedFailureDwarf()
    def test_decorator_xfail3(self):
        """Same as test_decorator_xfail, but with a custom decorator w/ no args"""
        if self.getDebugInfo() == "dwarf":
            self.fail()

    @add_test_categories(["dwarf"])
    def test_add_test_categories(self):
        """Test that add_test_categories limits the kinds of debug info test variants"""
        self.assertEqual(self.getDebugInfo(), "dwarf")

    @expectedFailureAll(compiler="fake", debug_info="dwarf")
    def test_decorator_xfail_all(self):
        """Test that expectedFailureAll requires all conditions to match to be xfail"""

    @skipIf(compiler="fake", debug_info="dwarf")
    def test_decorator_skip2(self):
        """Test that expectedFailureAll fails for the debug_info variant"""
        # Note: the following assertion would fail, if this were not skipped:
        # self.assertNotEqual(self.getDebugInfo(), "dwarf")

    @expectedFailureAll
    def test_xfail_empty(self):
        """Test that xfail can be empty"""
        self.fail()

    @expectedFailureAll(compiler=re.compile(".*"))
    def test_xfail_regexp(self):
        """Test that xfail can take a regex as a matcher"""
        self.fail()

    @expectedFailureAll(compiler=no_match(re.compile(".*")))
    def test_xfail_no_match(self):
        """Test that xfail can take a no_match matcher"""
        pass

    @expectedFailureIf(condition=True)
    def test_xfail_condition_true(self):
        self.fail()

    @expectedFailureIf(condition=False)
    def test_xfail_condition_false(self):
        pass



import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *


class TestPreRunLibraries(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(oslist=no_match(["darwin", "macos"]))
    def test(self):
        """Test that we find directly linked dylib pre-run."""

        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        # I don't know what the name of a shared library
        # extension is in general, so instead of using FindModule,
        # I'll iterate through the module and do a basename match.
        found_it = False
        for module in target.modules:
            file_name = module.GetFileSpec().GetFilename()
            if file_name.find("unlikely_name") != -1:
                found_it = True
                break

        self.assertTrue(
            found_it, "Couldn't find unlikely_to_occur_name in loaded libraries."
        )
